!---------------------------------------------------------------------------------------------------------!
    module globalvalues
      use precisions     
      implicit none

      

!--------------!      
!Number of types - do not change
!--------------!
      integer, parameter :: numtypes = 995
      integer, parameter :: numyears = 81
      



!--------------!
! Some details for the project
!--------------!
    integer, parameter :: whichModel = 4
    logical, parameter :: solveForBeta = .false.
    logical, parameter :: useInputtedBetaAsStartingPoint = .true.
    logical, parameter :: usePreviousEstimatedBeta = .true.
    logical, parameter :: noPenTaxAdv = .false.    
    logical, parameter :: everyoneHasHousing = .true.



    
    !--------------!      
    !! Some globals for approximation and simulation
    !--------------!      
    integer, parameter :: consGridSize = 150 ! When doing by grid search - how many consumption points
    logical, parameter  :: contEarn = .true. ! Continuous earnings - can't see much reason to change this
    integer, parameter :: optimMethod= 2 ! =1 for local search, =2 for (global) consumption on grid
    real (kind=rk), parameter :: uncondTrans = 1.0_rk ! Give people a pound every year to keep them away from zero
    logical, parameter :: solveUsingEuler = .true. ! true to maximize by finding root of Euler equation, false to maximize value function    
    logical, parameter :: neverUseGuess = .false. !do we want to disallow local optimzation from a guess based on the same node the subsequent period
    real (kind=rk), parameter :: guessPerturb   = 0.20_rk  !perturbation for the guess (initial search is in the range that is this proportion larger and smaller than the guess)   
    logical, parameter :: timeSaving = .false. ! For very coarse  grid - for debugging only
    logical :: callingFromVFI ! Sometimes subroutines need to know whether we're calling from solution(vfi) or simulation
    real(kind=rk) :: log2, numCycles1, numCycles2 ! These are used to speed to up linearinterp
    real (kind=rk), allocatable :: consGrid(:)
 
    
    !-----------------------------------------!  
    !Housing parameters
    !-----------------------------------------!      
    real (kind=rk), parameter :: housCG = 0.022857221_rk  !housing capital gain    
    real (kind=rk), parameter :: housYield = 0.070193222_rk !housing renral yield
    real (kind=rk), parameter :: mortR = 0.059 ! mortgage interest rate
    real (kind=rk), parameter :: portb0 =  0.05137295_rk,  portwealth = 0.00058849_rk, portwealth2 = -0.00000027782_rk !Share parameters for non pension wealth
    real (kind=rk), parameter :: lev0 = 1.1618549_rk, lev1 = -.02834176_rk, lev2 = .00017069_rk    !Leverage process
    
    !-----------------------------------------!      
    ! Some constants - do not change
    !-----------------------------------------!      
    integer, parameter  :: lastWorkAge = 64 ! last age of work
    integer, parameter  :: workChoices = 2 !extensive margin choice only
    integer, parameter  :: firstageatwhichdeath = 64 !no death until 64, happily
    integer, parameter  :: dbAge = 65 !age at which DB pension vests in  (this is only for the sensitivity in Appendix C), need to be paired with DB input text files
    integer, parameter  :: claimAge = 64 !age at which the DC pension is claimed
    real (kind=rk), parameter  :: bigdp  = 9999999999999.0_rk!A very large big positive real(kind=rk) number
    integer , parameter :: bigint = 99999!A very large big positive integer
 
    !--------------!      
    ! Some switches for sensitivity/robustness, results not in the paper - below is the default
    !--------------!          
    logical, parameter :: homogSurvProbs = .false. !no education specific survival probabilities
    logical, parameter :: doAveFixedEffect = .false. !everyone gets the average fixed effect
    logical, parameter :: unempOff = .false. ! unemployment risk is turn off
    logical, parameter :: useSurvProbs = .true. !if false everyone lives deterministically to the last peiod
    logical, parameter :: taxInterest = .false. ! taxatation of risk-free income
       
    !---------------------!      
    ! Some globals which will be set depending on value of whichModel
    !---------------------!            
     logical :: intRateOverRide, dcOverRide 
     real (kind=rk) :: intMeth , dcOverrideVal
      
    !---------------------------!      
    !Some globals that will be used in estimateion finding discount factor!
    !---------------------------!            
    real (kind=rk) :: cashData, penData, housingData, nonpenData, cashSim, penSim, housingSim, nonpenSim
    integer :: ageData
       
       
       
    contains
!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      
      subroutine getpaths(path_in, path_out)
        implicit none
        character (80), intent (out) :: path_in
        character (80), intent (out) :: path_out


    path_in = 'input\'
    path_out = 'output\'

      end subroutine getpaths

!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      

    
        
   

    end module globalvalues



