!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
    module globalvalues
      use precisions
      use types_generic
      implicit none


!--------------!      
!Number of types
!--------------!
      !integer, parameter :: numtypes = 996
      !integer, parameter :: numtypes = 200
      integer, parameter :: numtypes = 1
      integer, parameter :: numyears = 81
      
      integer leisurevec(3)
      data leisurevec/2, 1, 0/

!--------------!      
!Number of sims
!--------------!
      integer :: globalsims  
      
!--------------!      
!Constants
!--------------!      
      real (kind=rk) bigdp !A very large big positive real(kind=rk) number
      parameter (bigdp=9999999999999.0_rk)
      integer bigint !A very large big positive integer
      parameter (bigint=99999)

!--------------!      
! Some new globals, which I could put in points, but I'm trying not to
!--------------!      
      real (kind=rk), allocatable :: consGrid(:)

!--------------!      
! Which project
!--------------!      
   	  
   	  logical, parameter :: projectoptimality = .true.
   	  integer, parameter :: whichcomp = 2 ! =1 if desktop, =2 if Askella, =3 if Kajam
   	  
!--------------!      
!For estimation
!--------------!      
      
       
      
    contains
!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      
      subroutine getpaths(path_in, path_out)
        implicit none
        character (80), intent (out) :: path_in
        character (80), intent (out) :: path_out


if (whichcomp.eq.1) then
    path_in = 'C:\newassettest\data\'
    path_out = 'C:\newassettest\output\'
else if (whichcomp.eq.2) then
    path_in = 'P:\adandopt\data\modelinput\modelinput_a\'
    path_out = 'P:\adandopt\data\modeloutput\modeloutput_a\'
else if (whichcomp.eq.3) then
    path_in = 'P:\adandopt\data\modelinput\modelinput_k\'
    path_out = 'P:\adandopt\data\modeloutput\modeloutput_k\'
else if (whichcomp.eq.4) then
    path_in = 'P:\adandopt\data\modelinput\modelinput_h\'
    path_out = 'P:\adandopt\data\modeloutput\modeloutput_h\'
else if (whichcomp.eq.5) then
    path_in = 'P:\adandopt\data\modelinput\modelinput_t\'
    path_out = 'P:\adandopt\data\modeloutput\modeloutput_t\'
else if (whichcomp.eq.6) then
    path_in = 'P:\adandopt\data\modelinput\modelinput_r\'
    path_out = 'P:\adandopt\data\modeloutput\modeloutput_r\'
else
    write(*,*)  'whichcomp should be 1, 2, 3, 4, 5, or 6'
    stop    
end if        



      end subroutine getpaths

!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      


      subroutine readinglobaldata(firstageatwhichdeath, survprobs_m, survprobs_f, equivscale,numeqkids, numkids, dim)
        implicit none
        integer, intent (in) :: dim
        integer, intent (in) :: firstageatwhichdeath
        real (kind=rk), intent (out) :: survprobs_m(dim)
        real (kind=rk), intent (out) :: survprobs_f(dim)
        real (kind=rk), intent (out) :: equivscale(dim)
        real (kind=rk), intent (out) :: numeqkids(dim)
        real (kind=rk), intent (out) :: numkids(dim)

        integer :: ixtoday, todayage
        
        character (80) :: path_in
        character (80) :: path_out
        character (80) :: in_survprobs_m
        character (80) :: in_survprobs_f
        character (80) :: in_equivscale
        character (80) :: in_numeqkids
        
!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      
        call getpaths(path_in, path_out)
        in_survprobs_m = trim(path_in) // 'survprobs_m.txt'
        in_survprobs_f = trim(path_in) // 'survprobs_f.txt'
        in_equivscale = trim(path_in) // 'equivscale.txt'
        in_numeqkids = trim(path_in) // 'numeqkids.txt'
        
        open (unit=401, file=in_survprobs_m, action='read')
        read (401, *) survprobs_m
        close (401) 

        open (unit=402, file=in_survprobs_f, action='read')
        read (402, *) survprobs_f
        close (402) 

        open (unit=41, file=in_equivscale, action='read')
        read (41, *) equivscale
        close (41) 
		
        open (unit=42, file=in_numeqkids, action='read')
        read (42, *) numeqkids
        close (42) 


        ! Temporary
        ! This is not ideal in two ways - the switch for whether we're in project optimality for numkids of not is spread over two files
        ! Also I am using numeqkids * 5 for the number of kids
        ! To fix
        
        if (.not.(projectoptimality)) then
            numkids = numeqkids * 5        
        end if
 
        do ixtoday = 1, 81 , 1
          todayage = ixtoday + 19
          if (todayage.lt.firstageatwhichdeath) then
            survprobs_f(ixtoday) = 1.0_rk
            survprobs_m(ixtoday) = 1.0_rk
          end if                    
        end do
        !Definite death in last period
        survprobs_m(81) = 0.0_rk
        survprobs_f(81) = 0.0_rk

      end subroutine readinglobaldata

!I close the file so that the next time it is read, it will read from the beginning


!---------------------------------------------------------------------------------------------------------!
      subroutine readinpoints(allpoints, numkids, earnparams, opt_realised)
        implicit none
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
        type (grids), intent (out) :: allpoints(numtypes)
        real (kind=rk), intent(out) :: numkids(numyears, numtypes)
        real (kind=rk), intent(out), optional :: earnparams(6, numtypes)
        !real (kind=rk), intent(out), optional :: realisedearnings(numyears, numtypes)
        !integer , intent(out), optional :: realisedunemp(numyears, numtypes)
        type(typeopt_realisations), intent(inout), optional  :: opt_realised(numtypes)
!---------------------------------------------------------------------------------------------------------!

        integer :: ixtype
        character (80) :: path_in
        character (80) :: path_out
        character (80) :: in_points
        character (80) :: in_numkids
        character (80) :: in_earnparams
        character (80) :: in_fixedeffects
        character (80) :: in_growthrates
        character (80) :: in_realisedearnings
        character (80) :: in_realisedunemp 
        character (80) :: in_manshare
        character (80) :: in_hcons
        character (80) :: in_hcost
        character (80) :: in_statepen_m_f1
        character (80) :: in_s2p_m_f1
        character (80) :: in_statepen_m_f0
        character (80) :: in_s2p_m_f0
        character (80) :: in_statepen_f_m1
        character (80) :: in_s2p_f_m1
        character (80) :: in_statepen_f_m0
        character (80) :: in_s2p_f_m0
        character (80) :: in_dcgr
        
      
        real (kind=rk) :: readindata(148)
        integer :: ixdchoice
        
        integer :: typesforloop !will depend on whether we're in projectoptimality or otherwise
                
!Arguments
        call getpaths(path_in, path_out)

        in_points = trim(path_in) // 'points.txt'
        open (unit=110, file=in_points, action='read')

        in_numkids = trim(path_in) // 'numkids.txt'
        open (unit=111, file=in_numkids, action='read')

        in_earnparams = trim(path_in) // 'earnparams.txt'
        open (unit=112, file=in_earnparams, action='read')

        in_realisedearnings = trim(path_in) // 'realisedearnings.txt'
        open (unit=115, file=in_realisedearnings, action='read')

        in_realisedunemp = trim(path_in) // 'realisedunemp.txt'
        open (unit=116, file=in_realisedunemp, action='read')

        in_manshare = trim(path_in) // 'manshare.txt'
        open (unit=117, file=in_manshare, action='read')

        in_manshare = trim(path_in) // 'manshare.txt'
        open (unit=118, file=in_manshare, action='read')

        in_hcost = trim(path_in) // 'hhcost.txt'
        open (unit=119, file=in_hcost, action='read')

        in_hcons = trim(path_in) // 'hhcons.txt'
        open (unit=120, file=in_hcons, action='read')

        in_dcgr = trim(path_in) // 'dcgr.txt'
        open (unit=123, file=in_dcgr, action='read')
          
        in_statepen_m_f1 = trim(path_in) // 'statepen_m_f1.txt'
        open (unit=124, file=in_statepen_m_f1, action='read')

        in_s2p_m_f1 = trim(path_in) // 's2p_m_f1.txt'
        open (unit=125, file=in_s2p_m_f1, action='read')

        in_statepen_m_f0 = trim(path_in) // 'statepen_m_f0.txt'
        open (unit=126, file=in_statepen_m_f0, action='read')

        in_s2p_m_f0 = trim(path_in) // 's2p_m_f0.txt'
        open (unit=127, file=in_s2p_m_f0, action='read')

        in_statepen_f_m1 = trim(path_in) // 'statepen_f_m1.txt'
        open (unit=128, file=in_statepen_f_m1, action='read')

        in_s2p_f_m1 = trim(path_in) // 's2p_f_m1.txt'
        open (unit=129, file=in_s2p_f_m1, action='read')

        in_statepen_f_m0 = trim(path_in) // 'statepen_f_m0.txt'
        open (unit=130, file=in_statepen_f_m0, action='read')

        in_s2p_f_m0 = trim(path_in) // 's2p_f_m0.txt'
        open (unit=131, file=in_s2p_f_m0, action='read')


        if (projectoptimality) then
          typesforloop = 1
        else 
          typesforloop = numtypes
        end if

!For programme
        do ixtype = 1, typesforloop
         read (110, *) readindata
         allpoints(ixtype)%ass = int(readindata(1))
         allpoints(ixtype)%ass1 = int(readindata(2))
         allpoints(ixtype)%ass2 = int(readindata(3))
         allpoints(ixtype)%ass3 = int(readindata(4))
         allpoints(ixtype)%assgr1 = readindata(5)
         allpoints(ixtype)%assgr2 = readindata(6)
         allpoints(ixtype)%assgr3 = readindata(7)
               
         allpoints(ixtype)%dc = int(readindata(8))
         allpoints(ixtype)%dc1 = int(readindata(9))
         allpoints(ixtype)%dc2 = int(readindata(10))
         allpoints(ixtype)%dc3 = int(readindata(11))
         allpoints(ixtype)%dcgr1 = readindata(12)
         allpoints(ixtype)%dcgr2 = readindata(13)
         allpoints(ixtype)%dcgr3 = readindata(14)
        
          allpoints(ixtype)%cum = 1 !Hardcoding this as its now redundant
          allpoints(ixtype)%cum1 = 1
          allpoints(ixtype)%cum2 = 0
          allpoints(ixtype)%cum3 = 0
          allpoints(ixtype)%cumgr1 = 0.0_rk
          allpoints(ixtype)%cumgr2 = 0.0_rk
          allpoints(ixtype)%cumgr3 = 0.0_rk
          
          allpoints(ixtype)%pen = int(readindata(15))
          allpoints(ixtype)%pen1 = int(readindata(16))
          allpoints(ixtype)%pen2 = int(readindata(17))
          allpoints(ixtype)%pen3 = int(readindata(18))
          allpoints(ixtype)%pengr1 = readindata(19)
          allpoints(ixtype)%pengr2 = readindata(20)
          allpoints(ixtype)%pengr3 = readindata(21)
          allpoints(ixtype)%pengridbreak1 = readindata(22)
          allpoints(ixtype)%pengridbreak2 = readindata(23)
          allpoints(ixtype)%pengridlowerbound = readindata(24)
          allpoints(ixtype)%pengridupperbound = readindata(25)
          allpoints(ixtype)%ar1 = int(readindata(26))
          allpoints(ixtype)%ar1by2 = allpoints(ixtype)%ar1*2         
          allpoints(ixtype)%ar1by3 = allpoints(ixtype)%ar1*3    
          allpoints(ixtype)%ar1by4 = allpoints(ixtype)%ar1*4     
          allpoints(ixtype)%goldentol = 10**(readindata(27))
          allpoints(ixtype)%sims = int(readindata(28))
          allpoints(ixtype)%maxretage = int(readindata(29))
          allpoints(ixtype)%minretage = int(readindata(30))
          allpoints(ixtype)%baseretage = int(readindata(31))
          allpoints(ixtype)%maxstatepen = readindata(32)
          allpoints(ixtype)%claimch = int(readindata(33))
          allpoints(ixtype)%earlyretdisc = readindata(34)
          allpoints(ixtype)%dbdenom = readindata(35)
          allpoints(ixtype)%unemppay = readindata(36)          
          allpoints(ixtype)%probofdcdraw = readindata(37)
          allpoints(ixtype)%dcmeanret = readindata(38)
          allpoints(ixtype)%dcstdev = readindata(39)
          allpoints(ixtype)%annuityrate = readindata(40)
          allpoints(ixtype)%dclumpsumprop = readindata(41)
          allpoints(ixtype)%oneminusdclumpsumprop = 1.0_rk - readindata(41)
          allpoints(ixtype)%uncondtrans = readindata(42)
          allpoints(ixtype)%rate = readindata(43)
          allpoints(ixtype)%dcquad%dim = int(readindata(44))

          allpoints(ixtype)%dboff = readinlogical(readindata(45), 45)
          
          allpoints(ixtype)%dcoff = readinlogical(readindata(46), 46)
         
          allpoints(ixtype)%earn%params(1) = readindata(47)
          allpoints(ixtype)%earn%params(2) = readindata(48)
          allpoints(ixtype)%earn%params(3) = readindata(49)
          allpoints(ixtype)%earn%ar1mean = readindata(50)
          allpoints(ixtype)%earn%ar1rho = readindata(51)
          allpoints(ixtype)%earn%ar1stdev = readindata(52)

          !Get numdcchoices
          allpoints(ixtype)%numdcch = 1
          
          if (allpoints(ixtype)%dcoff) then
              allocate (allpoints(ixtype)%alldcchoice(allpoints(ixtype)%numdcch))
              allpoints(ixtype)%alldcchoice(1) = 0.0_rk
          else
              do ixdchoice = 2, 10 
                if (readindata(52+ixdchoice).gt.0.0001_rk) allpoints(ixtype)%numdcch = allpoints(ixtype)%numdcch + 1
              end do
                        
              allocate (allpoints(ixtype)%alldcchoice(allpoints(ixtype)%numdcch))
              do ixdchoice = 1, allpoints(ixtype)%numdcch
                allpoints(ixtype)%alldcchoice(ixdchoice) = readindata(52+ixdchoice)
              end do
           end if           
        allpoints(ixtype)%maxdccont = maxval(allpoints(ixtype)%alldcchoice)

        allpoints(ixtype)%tax2005%tfa = readindata(63)
        allpoints(ixtype)%tax2005%higherthreshold = readindata(64)
        allpoints(ixtype)%tax2005%basicrate = readindata(65)
        allpoints(ixtype)%tax2005%higherrate = readindata(66)
        allpoints(ixtype)%params%gs1 = readindata(67)
        allpoints(ixtype)%params%gs2 = readindata(68)
        allpoints(ixtype)%params%beta = readindata(69)
        allpoints(ixtype)%params%gamma = readindata(70)
        !If gamma ~=1 then I'll save this value to speed things up
         if (.not.((allpoints(ixtype)%params%gamma.gt.(1.0_rk-1.0d-2)).and.(allpoints(ixtype)%params%gamma.lt.(1.0_rk+1.0d-2)))) then
            allpoints(ixtype)%params%oneover1minusgamma =   1/(1-allpoints(ixtype)%params%gamma)
        end if 
         
        allpoints(ixtype)%pc%guarcredon = readinlogical(readindata(71), 71)
        allpoints(ixtype)%pc%capflowrate = readindata(72)
        allpoints(ixtype)%pc%capdisreg = readindata(73)
        allpoints(ixtype)%pc%eligage = readindata(74)
        allpoints(ixtype)%pc%level = readindata(75)

       
         allpoints(ixtype)%pc%savcredon = readinlogical(readindata(76), 76)
         allpoints(ixtype)%pc%sceligage = int(readindata(77))
         allpoints(ixtype)%pc%taper = readindata(78)
         allpoints(ixtype)%pc%oneminustaper = 1.0_rk - allpoints(ixtype)%pc%taper
         allpoints(ixtype)%pc%scthreshold = readindata(79)
         allpoints(ixtype)%pc%maxsavcred = (allpoints(ixtype)%pc%level  - &
                                            allpoints(ixtype)%pc%scthreshold) * allpoints(ixtype)%pc%oneminustaper 
         allpoints(ixtype)%pc%maxincforsavcred = allpoints(ixtype)%pc%level + &
                                                allpoints(ixtype)%pc%maxsavcred / allpoints(ixtype)%pc%taper

         allpoints(ixtype)%usesurvprob = readinlogical(readindata(80), 80)
         allpoints(ixtype)%unemprisk = readindata(81)
         allpoints(ixtype)%statepenage = int(readindata(82))
         allpoints(ixtype)%tax2005%statepenage = int(readindata(82)) !I store the state pension age in the tax system structure
         allpoints(ixtype)%maxworkage = int(readindata(83))       
         allpoints(ixtype)%params%gs0 = readindata(84)
       
         allpoints(ixtype)%params%separable  = readinlogical(readindata(85), 85, .true.)
         allpoints(ixtype)%params%quadratic  = readinlogical(readindata(85)-1, 85, .true.)
         allpoints(ixtype)%params%lmputil  = readinlogical(readindata(85)-2, 85, .true.)
         allpoints(ixtype)%params%fcwutil  = readinlogical(readindata(85)-3, 85, .true.)
         
         if ((allpoints(ixtype)%params%gamma.gt.(1.0_rk-1.0d-2)).and.(allpoints(ixtype)%params%gamma.lt.(1.0_rk+1.0d-2))) then
           allpoints(ixtype)%params%utilcostofworkifsep = log(readindata(86))
         else
           allpoints(ixtype)%params%utilcostofworkifsep = abs(((readindata(86))** &
         				(1-allpoints(ixtype)%params%gamma))*1/(1-allpoints(ixtype)%params%gamma))
         end if
         
         allpoints(ixtype)%dontchecksdfs  = readinlogical(readindata(87), 87)
         allpoints(ixtype)%dontdovalfuncsigncheck = readinlogical(readindata(88), 88)
        
         allpoints(ixtype)%params%cweight_base  = readindata(89)
         
         allpoints(ixtype)%usefairann = readinlogical(readindata(90), 90)
         allpoints(ixtype)%disastershock = readindata(91)
         allpoints(ixtype)%parttimeon = readinlogical(readindata(92), 92) 
         allpoints(ixtype)%parttimeutildisc = readindata(93)
         allpoints(ixtype)%parttimewagedisc = readindata(94)
         allpoints(ixtype)%useequivscale = readinlogical(readindata(95), 95)
         allpoints(ixtype)%params%fixedcost_base = readindata(96)
         allpoints(ixtype)%params%tothours_base = readindata(97)
         allpoints(ixtype)%params%fthours = readindata(98)
         allpoints(ixtype)%params%pthours = readindata(99)
         allpoints(ixtype)%params%anndelcweight = readindata(100)
         allpoints(ixtype)%params%anndelfixedcost = readindata(101)
         allpoints(ixtype)%params%anndeltothours = readindata(102)
         allpoints(ixtype)%ages2p = readindata(103)
         allpoints(ixtype)%s2pparam = readindata(104)
         allpoints(ixtype)%s2plel = readindata(105)
         allpoints(ixtype)%s2puel = readindata(106)
         allpoints(ixtype)%s2pon = readinlogical(readindata(107), 107)
         allpoints(ixtype)%ageforcequiv = readindata(108)
         allpoints(ixtype)%integrateoutsideoptimisation = readinlogical(readindata(109), 109)
         allpoints(ixtype)%findbalancedbudget = readinlogical(readindata(110), 110)
         allpoints(ixtype)%findthectomatchutil = readinlogical(readindata(111), 111)
         allpoints(ixtype)%ageforearningsgridstop = readindata(112)
         
         allpoints(ixtype)%labopt = 3
         allpoints(ixtype)%numyears = numyears
         allpoints(ixtype)%worktorule = .false.

         allpoints(ixtype)%numhhcomp = 3

         
         allpoints(ixtype)%pc%levelcoup = readindata(113)
         allpoints(ixtype)%pc%scthresholdcoup = readindata(114)
         
         allpoints(ixtype)%pc%maxsavcredcoup = (allpoints(ixtype)%pc%levelcoup  - &
                                            allpoints(ixtype)%pc%scthresholdcoup) * allpoints(ixtype)%pc%oneminustaper 
         allpoints(ixtype)%pc%maxincforsavcredcoup = allpoints(ixtype)%pc%levelcoup + &
                                                allpoints(ixtype)%pc%maxsavcredcoup / allpoints(ixtype)%pc%taper
                                                
         allpoints(ixtype)%spousespprop = readindata(115)
         allpoints(ixtype)%maxstatepencoup = allpoints(ixtype)%maxstatepen * (1.0_rk + allpoints(ixtype)%spousespprop) 
         allpoints(ixtype)%penpropinherit = readindata(116)
         allpoints(ixtype)%firstageatwhichdeath = readindata(117)
         allpoints(ixtype)%womanearnprop = readindata(118)
         if (allpoints(ixtype)%womanearnprop .lt.0.000000001_rk) then
             allpoints(ixtype)%nosepwomanearn = .true.
         else
             allpoints(ixtype)%nosepwomanearn = .false.
         end if             
         
         allpoints(ixtype)%calcequivscale = readinlogical(readindata(119), 119)
         if  (allpoints(ixtype)%nosepwomanearn) then
             allpoints(ixtype)%womannoworkprob(1) = 1.0_rk
             allpoints(ixtype)%womannoworkprob(2) = 1.0_rk         
         else
             allpoints(ixtype)%womannoworkprob(1) = readindata(120)
             allpoints(ixtype)%womannoworkprob(2) = readindata(121)
         end if
         
         allpoints(ixtype)%laboursupplyexogenous = readinlogical(readindata(122), 122) 
         allpoints(ixtype)%lastageofwork_exogls= readindata(123)
         allpoints(ixtype)%sparefordev = readindata(124) 
         allpoints(ixtype)%params%multFelByEquiv = readinlogical(readindata(125), 125) 
         allpoints(ixtype)%params%bequestOn = readinlogical(readindata(126), 126) 
         allpoints(ixtype)%params%bequestT = readindata(127)
         allpoints(ixtype)%params%bequestK = readindata(128)
         allpoints(ixtype)%tax2005%childBenRate1 = readindata(129)
         allpoints(ixtype)%tax2005%childBenRate2 = readindata(130)
         allpoints(ixtype)%exogH = readinlogical(readindata(131), 131) 
         allpoints(ixtype)%tax2005%lel = readindata(132)
         allpoints(ixtype)%tax2005%uel = readindata(133)
         allpoints(ixtype)%tax2005%NIlower = readindata(134)
         allpoints(ixtype)%tax2005%NIhigher = readindata(135)
         allpoints(ixtype)%tax2005%startinglimit = readindata(136)
         allpoints(ixtype)%tax2005%startingrate = readindata(137)      
         allpoints(ixtype)%exogSPens = readinlogical(readindata(138), 138)
         allpoints(ixtype)%tax2005%tfa6574 = readindata(139)      
         allpoints(ixtype)%tax2005%tfa75plus = readindata(140)              
         allpoints(ixtype)%exogDC = readinlogical(readindata(141), 141)               
         allpoints(ixtype)%tax2005%taxInterest = readinlogical(readindata(142), 142)              
         allpoints(ixtype)%neverInterpOnPolicy = readinlogical(readindata(143), 143)              
         allpoints(ixtype)%annAdminLoad = readindata(144)
         allpoints(ixtype)%equivHousingFlow  = readinlogical(readindata(145), 145)         
         allpoints(ixtype)%multOnHousingFlow = readindata(146)
         allpoints(ixtype)%lastWorkAgeOpt = readindata(147)         
         allpoints(ixtype)%cons = 100         
         allpoints(ixtype)%optimMethod = readindata(148)
       end do  
         !allpoints(1)%optimMethod = 1
         !allpoints(2)%optimMethod = 2
         
       

        !These are housing costs and housing consumption - only necessary for project optimality. But leaving them here for the moment
        !as in vfi and simulate I haven't conditioned on that. Maybe should use hhexog to control whether to bring them in or not
    
        if (projectoptimality) then         
           if (.not.(present(opt_realised))) then      
              write(*,*) 'error in calling globalvalues. projectoptimality is on, and opt_realised is not included as an argument'
           end if
           
           do ixtype = 1, numtypes
                !Now read in number of children          
                read (111, *) numkids(:, ixtype)
                if (present(earnparams)) then
                   read (112, *) earnparams(:, ixtype)
                end if

                !allocate (opt_realised(ixtype)%realisedearnings(numyears))
                !allocate (opt_realised(ixtype)%realisedunemp(numyears))
                !allocate (opt_realised(ixtype)%manshare(numyears))
                !allocate (opt_realised(ixtype)%hcons(numyears))
                !allocate (opt_realised(ixtype)%hcost(numyears))
                !
                
                    
                !Now read in a vector of earnings parameters for each individual        
                read (115, *) opt_realised(ixtype)%realisedearnings               
                read (116, *) opt_realised(ixtype)%realisedunemp
                read (117, *) opt_realised(ixtype)%manshare
                read (119, *) opt_realised(ixtype)%hcost
                read (120, *) opt_realised(ixtype)%hcons
                
                !read (121, *) opt_realised(ixtype)%statepen
                !read (122, *) opt_realised(ixtype)%s2p
                read (123, *) opt_realised(ixtype)%dcgr
                read (124, *) opt_realised(ixtype)%statepen_m_f1
                read (125, *) opt_realised(ixtype)%s2p_m_f1
                read (126, *) opt_realised(ixtype)%statepen_m_f0
                read (127, *) opt_realised(ixtype)%s2p_m_f0
                read (128, *) opt_realised(ixtype)%statepen_f_m1
                read (129, *) opt_realised(ixtype)%s2p_f_m1
                read (130, *) opt_realised(ixtype)%statepen_f_m0
                read (131, *) opt_realised(ixtype)%s2p_f_m0
   
                !Read in manshare into points too - but in a different channel to avoid issues with opening and closing
                allocate (allpoints(ixtype)%manshare(numyears))
                read (118, *) allpoints(ixtype)%manshare                  
           end do !ixtype

            do ixtype = 2, numtypes
              allpoints(ixtype) = allpoints(1)
            end do
        else if (.not.(projectoptimality)) then
            read (119, *) opt_realised(1)%hcost
            read (120, *) opt_realised(1)%hcons
            read (123, *) opt_realised(1)%dcgr
            
            do ixtype = 2, numtypes
               opt_realised(ixtype)%hcost = opt_realised(1)%hcost
               opt_realised(ixtype)%hcons = opt_realised(1)%hcons              
               opt_realised(ixtype)%dcgr = opt_realised(1)%dcgr
            end do               
      end if


       
        call checkpoints
      contains
        
        logical function readinlogical(realoneorzero, whichvalinallpoints, oneistrueallothersfalse)
          implicit none
          
          real (kind=rk) :: realoneorzero
          integer :: whichvalinallpoints
          logical, optional :: oneistrueallothersfalse
          logical :: inprogoneistrueallothersfalse
          
          if (present(oneistrueallothersfalse)) then
          	inprogoneistrueallothersfalse = oneistrueallothersfalse
          else 
          	inprogoneistrueallothersfalse = .false.
          end if
          
          if (inprogoneistrueallothersfalse) then
             if ((realoneorzero.gt.0.999_rk) .and. (realoneorzero.lt.1.001_rk)) then
                readinlogical  = .true.
              else
                readinlogical = .false.
              end if
          else     
              if ((realoneorzero.gt.-0.001_rk) .and. (realoneorzero.lt.0.001_rk)) then
                readinlogical = .false.
              else if ((realoneorzero.gt.0.999_rk) .and. (realoneorzero.lt.1.001_rk)) then
                readinlogical  = .true.
              else
                write (*, *) 'error in readindata - an element should be either zero or 1'
                write (*, *) 'element is', whichvalinallpoints
                stop
              end if
          end if
          
        end function readinlogical

        subroutine checkpoints
          implicit none

          do ixtype = 1, numtypes
            if ((allpoints(ixtype)%dboff) .and. (allpoints(ixtype)%cum.ne.1)) then
              write (*, *) 'Warning'
              write (*, *) 'dboff is set to .true. and cumulgridpoints is not set to 1'
              write (*, *) 'user-provided cumulgridpoints has been reset to 1'
              allpoints(ixtype)%cum = 1
              allpoints(ixtype)%cum1 = 1
              allpoints(ixtype)%cum2 = 0
              allpoints(ixtype)%cum3 = 0
            end if

            if ((allpoints(ixtype)%dcoff) .and. (allpoints(ixtype)%dc.ne.1)) then
              write (*, *) 'Warning'
              write (*, *) 'dcoff is set to .true. and dcpoints is not set to 1'
              write (*, *) 'user-provided dcgridpoints has been reset to 1'
              allpoints(ixtype)%dc = 1
              allpoints(ixtype)%dc1 = 1
             allpoints(ixtype)%dc2 = 0
              allpoints(ixtype)%dc3 = 0
              allpoints(ixtype)%dcquad%dim = 1
            end if

            if ((allpoints(ixtype)%dcoff) .and. (allpoints(ixtype)%dboff)) then
              if ((allpoints(ixtype)%maxretage.ne.(numyears+19)) .or. (allpoints(ixtype)%minretage.ne.(numyears+ &
                  19))) then
                write (*, *) 'Warning'
                write (*, *) 'Type', ixtype
                write (*, *) 'db and dc are off and either maxretage or minretage is not equal to highest age'
                write (*, *) 'maxretage or minretage have been set equal to the highestage'
                allpoints(ixtype)%maxretage = (numyears+19)
                allpoints(ixtype)%minretage = (numyears+19)
              end if
            end if

            if (allpoints(ixtype)%alldcchoice(allpoints(ixtype)%numdcch).ne.allpoints(ixtype)%maxdccont) then
              write (*, *) 'Warning'
              write (*, *) 'Type', ixtype
              write (*, *) 'The top point in dc menu is not equal to specified maxdccont.'
              write (*, *) 'The latter has been set to the former'
              allpoints(ixtype)%maxdccont = allpoints(ixtype)%alldcchoice(allpoints(ixtype)%numdcch)
            end if

            if ( (.not.(allpoints(ixtype)%s2pon)).and.(allpoints(ixtype)%ages2p.le.allpoints(ixtype)%maxretage)) then
              allpoints(ixtype)%ages2p = 100
              write (*, *) 'Warning'
              write (*, *) 'Type', ixtype
              write (*, *) 'S2p is off but age2p is less than maxretage'
              write (*, *) 'This is very inefficient' 
              write(*,*) 'ages2p has been set to 100'
            end if   

            if (allpoints(ixtype)%ages2p.lt.allpoints(ixtype)%minretage) then
              allpoints(ixtype)%ages2p = 100
              write (*, *) 'Warning'
              write (*, *) 'Type', ixtype
              write (*, *) 'Ages2p is less than minretage. The programme does not allow for this'
               stop
            end if   
    
            if ((allpoints(ixtype)%pc%savcredon).and.( allpoints(ixtype)%pc%scthreshold.gt.allpoints(ixtype)%pc%level)) then
              write (*, *) 'Error'
              write (*, *) 'Type', ixtype
              write (*, *) 'Saving Threshold is greater than pension credit level and savings credit is turned on'
              stop            
            end if

          end do
        end subroutine checkpoints



      end subroutine readinpoints
!----------------------------------------------------------------------------------------------------------------!      

!----------------------------------------------------------------------------------------------------------------!      
!----------------------------------------------------------------------------------------------------------------!      

    end module globalvalues



