    module incomeprocess
      use precisions
      use types_generic

      implicit none


    contains
      subroutine getprocesses(numyears, earn, ar1points, lastAgeForStoch, detincome, shocks, pmat, cdf)

        implicit none
        integer, intent (in) :: numyears
        type (earnproc), intent (in) :: earn
        integer, intent (in) :: ar1points
        integer, intent (in) :: lastAgeForStoch

!Arguments to spit out
        real (kind=rk), intent (out) :: detincome(numyears)

        real (kind=rk), intent (out) :: shocks(ar1points, numyears)
        real (kind=rk), intent (out) :: pmat(ar1points, ar1points, numyears)
        real (kind=rk), intent (out) :: cdf(ar1points, ar1points, numyears)

!For programme
        real (kind=rk) :: logdetincome(numyears)
        real (kind=rk) :: logshocks(ar1points, numyears)

        real (kind=rk) :: var(numyears)
        real (kind=rk) :: stdev(numyears)
        real (kind=rk) :: logar1bounds(ar1points+1, numyears)

        integer :: age

        integer :: ifail
        real (kind=rk) :: todnormlowerbound
        real (kind=rk) :: todnormupperbound
        real (kind=rk) :: sumforcdf

        real (kind=rk) :: weights(64)
        real (kind=rk) :: abscis(64)
        real (kind=rk) :: runningcount
        real (kind=rk), parameter :: pi = 3.14159265
 

!Indexes
        integer ixyear
        integer ixorder
        integer ixnode_yest
        integer ixnode_tod
        integer ixquad

!NAG
        real (kind=rk) :: g01eaf

!-------------------------------!
!First get deterministic income
!-------------------------------!
        do ixyear = 1, numyears
          logdetincome(ixyear) = earn%params(1) !Constant
          age = ixyear + 19
          do ixorder = 2, 3
            logdetincome(ixyear) = logdetincome(ixyear) + earn%params(ixorder)*(dble(age)**(ixorder-1))
          end do !ixyear
        end do !ixyears  

!--------------------------------------------------!
!Now, get a grid that is (gridpoints, numyears) 
!--------------------------------------------------!

!First get the variances and standard deviations of the shock each year
        do ixyear = 1, numyears
          if (ixyear.eq.1) then
            var(ixyear) = earn%ar1stdev*earn%ar1stdev
            stdev(ixyear) = earn%ar1stdev
          else
            var(ixyear) = earn%ar1rho*earn%ar1rho*var(ixyear-1) + earn%ar1stdev*earn%ar1stdev
            stdev(ixyear) = sqrt(var(ixyear))
          end if
        end do !ixyears  

!Now get the shocks vectors
        do ixyear = 1, numyears, 1
          call getshockvector(stdev(ixyear), ar1points, logshocks(:,ixyear), logar1bounds(:,ixyear))
        end do

!Now get the transition probabilities
!First year 1 - unconditional distribtion
        do ixnode_yest = 1, ar1points, 1
          do ixnode_tod = 1, ar1points, 1
            pmat(ixnode_yest, ixnode_tod, 1) = 1.0/dble(ar1points)
          end do
        end do

        do ixyear = 2, numyears, 1
          do ixnode_yest = 1, ar1points, 1
!First do for the lowest shock tomorrow
          todnormupperbound = (logar1bounds(2,ixyear)-earn%ar1rho*logshocks(ixnode_yest,ixyear-1))/earn%ar1stdev
          pmat(ixnode_yest, 1, ixyear) = g01eaf('L', todnormupperbound, ifail) - 0.0

          !New way
          call getquadweightsbounded(64, logar1bounds(ixnode_yest,ixyear-1), logar1bounds(ixnode_yest+1,ixyear-1), &
                    weights, abscis)
          
          runningcount = 0.0_rk
          do ixquad = 1, 64
            runningcount = runningcount + weights(ixquad)*forintegration(abscis(ixquad), logar1bounds((2), ixyear)&
            , logar1bounds(1, ixyear), stdev(ixyear-1), earn%ar1stdev, earn%ar1rho)
          end do
          runningcount = runningcount * ar1points/((2.0_rk * pi * (stdev(ixyear-1)**2))**(0.5))

          pmat(ixnode_yest, 1, ixyear) = runningcount        
!Now do the second to the second last
            do ixnode_tod = 2, ar1points - 1, 1
              !Old way
              todnormupperbound = (logar1bounds((ixnode_tod+1),ixyear)-earn%ar1rho*logshocks(ixnode_yest,ixyear-1))/earn &
                %ar1stdev
              todnormlowerbound = (logar1bounds(ixnode_tod,ixyear)-earn%ar1rho*logshocks(ixnode_yest,ixyear-1))/ &
                earn%ar1stdev
              pmat(ixnode_yest, ixnode_tod, ixyear) = g01eaf('L', todnormupperbound, ifail) - &
                g01eaf('L', todnormlowerbound, ifail)
                
              !New way
              call getquadweightsbounded(64, logar1bounds(ixnode_yest,ixyear-1), logar1bounds(ixnode_yest+1,ixyear-1), &
                        weights, abscis)
              
              runningcount = 0.0_rk
              do ixquad = 1, 64
                runningcount = runningcount + weights(ixquad)*forintegration(abscis(ixquad), logar1bounds((ixnode_tod+1), ixyear)&
                , logar1bounds(ixnode_tod, ixyear), stdev(ixyear-1), earn%ar1stdev, earn%ar1rho)
              end do
              runningcount = runningcount * ar1points/((2.0_rk * pi * (stdev(ixyear-1)**2))**(0.5))
              pmat(ixnode_yest, ixnode_tod, ixyear) = runningcount        
            end do
            
!Now do the highest shock tomorrow
            todnormlowerbound = (logar1bounds(ar1points,ixyear)-earn%ar1rho*logshocks(ixnode_yest,ixyear-1))/ &
              earn%ar1stdev
            pmat(ixnode_yest, ar1points, ixyear) = 1 - g01eaf('L', todnormlowerbound, ifail)
            
            !New way
              call getquadweightsbounded(64, logar1bounds(ixnode_yest,ixyear-1), logar1bounds(ixnode_yest+1,ixyear-1), &
                        weights, abscis)
              
              runningcount = 0.0_rk
              do ixquad = 1, 64
                runningcount = runningcount + weights(ixquad)*forintegration(abscis(ixquad), &
                logar1bounds((ar1points+1), ixyear), logar1bounds(ar1points, ixyear), &
                stdev(ixyear-1), earn%ar1stdev, earn%ar1rho)
           
              end do
              
              runningcount = runningcount * &
              	ar1points/((2.0_rk * pi * (stdev(ixyear-1)**2))**(0.5))                                 
             
              pmat(ixnode_yest, ar1points, ixyear) = runningcount 
          end do
        end do

    ! Replace the transitions probabilities with 1s and zeros for years after last age
   do ixyear = lastAgeForStoch - 19 + 1, numyears, 1
        pmat(:, :, ixyear) = 0
        do ixnode_tod = 1, ar1points
            pmat(ixnode_tod, ixnode_tod, ixyear) = 1.0_rk          
        end do
   end do

!Now just ensure probabilities sum EXACTLY to one but deliver an error message if the sum is too far from 1
        do ixyear = 1, numyears, 1
          do ixnode_yest = 1, ar1points
            if ((sum(pmat(ixnode_yest,:,ixyear)).lt.0.98) .or. (sum(pmat(ixnode_yest,:,ixyear)).gt.1.02)) then
              write (*, *) 'error in incomeprocess: probabilities do not sum to 1'
              write (*, *) ' at', ixnode_yest, ':', ixyear, 'sum(pmat(ixnode_yest,:, ixyear)) = ', &
                sum(pmat(ixnode_yest,:,ixyear))
              stop
            end if
            pmat(ixnode_yest, :, ixyear) = pmat(ixnode_yest, :, ixyear)/sum(pmat(ixnode_yest,:,ixyear))
          end do

!Get CDF
          do ixnode_yest = 1, ar1points
            sumforcdf = 0
            do ixnode_tod = 1, ar1points
              sumforcdf = sumforcdf + pmat(ixnode_yest, ixnode_tod, ixyear)
              cdf(ixnode_yest, ixnode_tod, ixyear) = sumforcdf
            end do
          end do

        end do !ixyear


!The deterministic component and the shocks are in logs. I now turn them into levels
        detincome = exp(logdetincome)
        shocks = exp(logshocks)
contains

! ---------------------------------------------------------------------------------------------------------!
     subroutine getquadweightsbounded(quadpoints, lowerlimit, upperlimit, weights, abscis)
        implicit none

        integer, intent (in) :: quadpoints
        real (kind=rk), intent (out) :: weights(quadpoints), abscis(quadpoints)
        real (kind=rk), intent(in) :: lowerlimit, upperlimit
        
           ! For NAG MARK 26 - OUT  NAG MARK 24 - IN
        !external d01baz !Pre mark 24
        integer :: itype, ifail


          ifail = 0
        
          
          !----------------------------------------------------------!
           ! For NAG MARK 26 - OUT  NAG MARK 24 - IN
            !!          
              !itype = 0
              !call d01bbf(d01baz, lowerlimit, upperlimit, itype, quadpoints, weights, abscis, ifail)
            !!
          !----------------------------------------------------------!
              
          
          !----------------------------------------------------------!          
           ! For NAG MARK IN - OUT  NAG MARK 24 - OUT
            !! 
                call d01tbf(0,lowerlimit, upperlimit, quadpoints,weights, abscis, ifail)
            !!
          !----------------------------------------------------------!
          
          

      end subroutine getquadweightsbounded
!-----------------------------------------------------------------------------------------------------------!
!-----------------------------------------------------------------------------------------------------------!
!This is the integrand in the middle of page 58 of Adda and Cooper 
      real (kind=rk) function forintegration(u, epsilonjplus1, epsilonj,sigmaepsilon, sigma, rho)
        implicit none
        real (kind=rk) :: u
        real (kind=rk) :: epsilonjplus1
        real (kind=rk) :: epsilonj
        real (kind=rk) :: sigmaepsilon
        real (kind=rk) :: sigma
        real (kind=rk) :: rho
        integer :: ifail
        
        real (kind=rk) :: scratch1
        real (kind=rk) :: scratch2
        
        
        ifail = 0
        scratch1 = g01eaf('L', (epsilonjplus1 - rho*u)/sigma, ifail)
        scratch2 = g01eaf('L', (epsilonj - rho*u)/sigma, ifail)
         
        forintegration = exp(-(u**2)/(2*(sigmaepsilon**2)))* &
                           (g01eaf('L', (epsilonjplus1 - rho*u)/sigma, ifail) -  g01eaf('L', (epsilonj - rho*u)/sigma, ifail)) 
        
        
        
      end function forintegration
! ---------------------------------------------------------------------------------------------------------!

      end subroutine getprocesses


!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      subroutine getshockvector(sigma, gridpoints, zvect, evect)
        implicit none

!Arguments
        real (kind=rk), intent (in) :: sigma
        integer, intent (in) :: gridpoints
        real (kind=rk), intent (out) :: zvect(gridpoints)
        real (kind=rk), intent (out) :: evect(gridpoints+1)

!NAG
        external g01faf
        real (kind=rk) g01faf

!For programme
        integer ixi, ifail

!Finding the points that divide the distribution into N segments 
!First set -inf and inf. Setting them as +/- five times standard deviations
        evect(1) = -4.0_rk*sigma
        evect(gridpoints+1) = 4.0_rk*sigma

!Now recursively get the rest of the points
!Formula top of page 58 in Adda and Cooper)
!Formula below 3.10 on page 63 of the draft
        do ixi = 2, gridpoints
          ifail = 0
          evect(ixi) = sigma*g01faf('L', (dble((ixi-1))/gridpoints), ifail)
        end do



!Now check that the plus and minus inf care actually low and high enough to bound these. 
!I don't want to set them too high or it causes problems with the integration
        if ((evect(1).ge.evect(2)) .or. (evect(gridpoints+1).le.evect(gridpoints))) then
          write (*, *) 'error in incomeprocess evect(1) not low enough or evect(argridpoints+1) not high enough'
          stop
        end if

!Finding the expected value within each interval 
!Third formula on page 58  in Adda and Cooper
        do ixi = 1, gridpoints
          zvect(ixi) = gridpoints*sigma*(norm_pdf((evect(ixi))/sigma)-norm_pdf((evect(ixi+1))/sigma))
        end do

      contains
!---------------------------------------------------------------------------------------------------------------!     
        function norm_pdf(x)
          implicit none
          real (kind=rk) x, norm_pdf, pi
          parameter (pi=3.14159265)

          norm_pdf = 1.0_rk/sqrt(2.0_rk*pi)*exp(-(x)**2/2.0_rk)
        end function norm_pdf

      end subroutine getshockvector
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------------!          
  subroutine addunemptoincome(points, ar1shocksnoue, ar1pmatnoue, ar1cdfnoue, ar1shockswithue, ar1pmatwithue, ar1cdfwithue)
   implicit none
    type (grids), intent (in) :: points
    real (kind=rk), intent (in) :: ar1shocksnoue(points%ar1,points%numyears)
    real (kind=rk), intent (in) :: ar1pmatnoue(points%ar1,points%ar1,points%numyears)
    real (kind=rk), intent (in) :: ar1cdfnoue(points%ar1,points%ar1,points%numyears)
    real (kind=rk), intent (out) :: ar1shockswithue(points%ar1by2,points%numyears)
    real (kind=rk), intent (out) :: ar1pmatwithue(points%ar1by2,points%ar1by2,points%numyears)
    real (kind=rk), intent (out) :: ar1cdfwithue(points%ar1by2,points%ar1by2,points%numyears)
        
    integer :: ixar1node1, ixtoday
    
    do ixtoday = 1, points%numyears, 1
        !The shocks
        do ixar1node1 = 1, points%ar1, 1
            ar1shockswithue(ixar1node1, ixtoday) = ar1shocksnoue(ixar1node1, ixtoday)
        end do
        do ixar1node1 = (points%ar1 + 1), points%ar1by2, 1
            ar1shockswithue(ixar1node1, ixtoday) = 0.0_rk
        end do
       
        ! PDF
        do ixar1node1 = 1, points%ar1, 1
           ar1pmatwithue(ixar1node1,1:points%ar1,ixtoday) =(1.0_rk - points%unemprisk)*ar1pmatnoue(ixar1node1,:,ixtoday)
           ar1pmatwithue(ixar1node1,points%ar1+1:points%ar1by2,ixtoday) = points%unemprisk*ar1pmatnoue(ixar1node1,:,ixtoday)  
        end do
        do ixar1node1 = 1, points%ar1, 1
            ar1pmatwithue(points%ar1+ixar1node1,:,ixtoday) =  ar1pmatwithue(ixar1node1,:,ixtoday)
        end do
        
        ! CDF
        do ixar1node1 = 1, points%ar1, 1
           ar1cdfwithue(ixar1node1,1:points%ar1,ixtoday) = (1.0_rk - points%unemprisk)*ar1cdfnoue(ixar1node1,:,ixtoday)
           ar1cdfwithue(ixar1node1,points%ar1+1:points%ar1by2,ixtoday) = (1.0_rk - points%unemprisk) + &
            points%unemprisk*ar1cdfnoue(ixar1node1,:,ixtoday)  
        end do
           ar1cdfwithue(points%ar1+1:points%ar1by2,:,ixtoday) =  ar1cdfwithue(1:points%ar1,:,ixtoday)
       
    end do         
    
end subroutine addunemptoincome

!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------------!
  subroutine adduneandwomentoincome(points, ar1shocksnoue, ar1pmatnoue, ar1cdfnoue, ar1shockswithueandw, ar1pmatwithueandw, ar1cdfwithueandw)
   use routines_generic
   implicit none
   
    type (grids), intent (in) :: points
    real (kind=rk), intent (in) :: ar1shocksnoue(points%ar1,points%numyears)
    real (kind=rk), intent (in) :: ar1pmatnoue(points%ar1,points%ar1,points%numyears)
    real (kind=rk), intent (in) :: ar1cdfnoue(points%ar1,points%ar1,points%numyears)
    real (kind=rk), intent (out) :: ar1shockswithueandw(points%ar1by4,points%numyears)
    real (kind=rk), intent (out) :: ar1pmatwithueandw(points%ar1by4,points%ar1by4,points%numyears)
    real (kind=rk), intent (out) :: ar1cdfwithueandw(points%ar1by4,points%ar1by4,points%numyears)
        
    integer :: ixar1node1, ixtoday
    integer :: ar1by3
    
    real (kind=rk) :: ar1shockswithue(points%ar1by2,points%numyears)
    real (kind=rk) :: ar1pmatwithue(points%ar1by2,points%ar1by2,points%numyears)
    real (kind=rk) :: ar1cdfwithue(points%ar1by2,points%ar1by2,points%numyears)

    real (kind=rk) :: cdf(points%ar1by4)
    
    ar1by3 = points%ar1  * 3
    
    
    do ixtoday = 1, points%numyears, 1
        !New way of doing the shocks - the shock is the same in each of the four blocks. I map it into a different earnings level in
        !getearningsarry
        
        ar1shockswithueandw(1:points%ar1, ixtoday) = ar1shocksnoue(:, ixtoday) 
        ar1shockswithueandw((points%ar1 + 1):points%ar1by2, ixtoday) =  ar1shocksnoue(:, ixtoday) 
        ar1shockswithueandw((points%ar1by2 + 1):points%ar1by3, ixtoday) = ar1shocksnoue(:, ixtoday) 
        ar1shockswithueandw((points%ar1by3 + 1):points%ar1by4, ixtoday) = ar1shocksnoue(:, ixtoday) 

       
       ! PDF
        !First a transition for men, ignoring female labour supply
        do ixar1node1 = 1, points%ar1, 1 !First conditional on the man being in work today
           ar1pmatwithue(ixar1node1,1:points%ar1,ixtoday) =(1.0_rk - points%unemprisk)*ar1pmatnoue(ixar1node1,:,ixtoday)
           ar1pmatwithue(ixar1node1,points%ar1+1:points%ar1by2,ixtoday) = points%unemprisk*ar1pmatnoue(ixar1node1,:,ixtoday)  
        end do
        do ixar1node1 = 1, points%ar1, 1 !Next conditional on the man not being in work today
            ar1pmatwithue(points%ar1+ixar1node1,:,ixtoday) =  ar1pmatwithue(ixar1node1,:,ixtoday)
        end do
        
        ! Now embed to take account of women's work
        do ixar1node1 = 1, points%ar1by2, 1 !From a state of women being in work
           ar1pmatwithueandw(ixar1node1,1:points%ar1by2,ixtoday) =(1.0_rk -  points%womannoworkprob(1))*ar1pmatwithue(ixar1node1,:,ixtoday)
           ar1pmatwithueandw(ixar1node1,points%ar1by2+1:points%ar1by4,ixtoday) = points%womannoworkprob(1)*ar1pmatwithue(ixar1node1,:,ixtoday)  
        end do
        do ixar1node1 = 1, points%ar1by2, 1 !From a state of women being out of work
           ar1pmatwithueandw(points%ar1by2+ixar1node1,1:points%ar1by2,ixtoday) =(1.0_rk -  points%womannoworkprob(2))*ar1pmatwithue(ixar1node1,:,ixtoday)
           ar1pmatwithueandw(points%ar1by2+ixar1node1,points%ar1by2+1:points%ar1by4,ixtoday) =  points%womannoworkprob(2)*ar1pmatwithue(ixar1node1,:,ixtoday)  
        end do
                
        
        ! CDF
        do ixar1node1 = 1, points%ar1by4, 1
             call pdftocdf(points%ar1by4, ar1pmatwithueandw(ixar1node1,:,ixtoday), cdf)
             ar1cdfwithueandw(ixar1node1,:,ixtoday) = cdf
        end do   
        
       
    end do         
    
end subroutine adduneandwomentoincome


!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------------!
subroutine suppressfanningoutofincome(ar1points, numyears, detincome, shocks, pmat, cdf, lastWorkAge)
    implicit none
        
        integer, intent(in) :: ar1points
        integer, intent(in) :: numyears
        real (kind=rk), intent (inout) :: detincome(numyears)
        real (kind=rk), intent (inout) :: shocks(ar1points, numyears)
        real (kind=rk), intent (inout) :: pmat(ar1points, ar1points, numyears)
        real (kind=rk), intent (inout) :: cdf(ar1points, ar1points, numyears)
        integer, intent(in) :: lastWorkAge
        
        !For programme 
        integer :: ixtoday, todayage
        
        do ixtoday = 1, numyears, 1
          todayage = ixtoday + 19
          if (todayage.gt.lastWorkAge) then
            shocks(:, ixtoday) = shocks(:, ixtoday - 1)
            pmat(:,:, ixtoday) = pmat(:,:, ixtoday - 1)
            cdf(:,:, ixtoday) =  cdf(:,:, ixtoday-1)
          end if !(todayage.gt.ageforsuppressing)
            
        end do !ixtoday
        
        
        
end subroutine suppressfanningoutofincome
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------------!          

    end module incomeprocess
