! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
    program main
      use globalvalues
      use precisions
      use routines_specific
      use routines_generic
      use incomeprocess
      use pension
      use types


      implicit none


! ----------------------------------------------------------------------!
! Declarations														   !
! ----------------------------------------------------------------------!
! For income processes
      real (kind=rk) :: detincome(numyears)
     
      real (kind=rk), allocatable, dimension(:,:) :: ar1shocks
      real (kind=rk), allocatable, dimension(:,:, :) :: ar1pmat, ar1cdf, allearnings

! Grids
      real (kind=rk), allocatable, dimension(:,:) :: assgrid, dcgrid,housgrid
      real (kind=rk), allocatable, dimension(:) :: pengrid

! Parameters
      type (grids) :: points
      type (grids) :: allpoints(numtypes)

! For holding results of vfi (pol function etc.)
      type (policy_funcs) :: policy
      type (val_funcs) :: exval
      integer :: sdffail(4)
      real (kind=rk) :: sdffailprop(4)

! For holding result of simualte
      real (kind=rk), dimension(4) :: failmaxoversims, failaveoversims
      type (seriesstats) :: alltypstats(numtypes)
      real (kind=rk), dimension(numtypes) :: expdiscutil

      
! For time
      real (kind=rk) :: time1, time2
      real (kind=rk) :: timetypestart, timetypeend
      real (kind=rk) :: timealltypes(numtypes)


! For program
      integer :: ixtype
      logical :: firstcall
      logical :: doprintmoments 

! To sort
      real (kind=rk) :: estmoment   
       real (kind=rk), allocatable :: ar1shocksnoue2(:,:)
       real (kind=rk), allocatable :: ar1pmatnoue2(:,:,:)
       real (kind=rk), allocatable :: ar1cdfnoue2(:,:,:)
       
 
       real (kind=rk), allocatable :: allcpaths(:,:)
       integer, allocatable :: alllpaths(:,:)
       integer, allocatable :: allhhcomps(:,:)
       logical :: doprintsimdetail
       real (kind=rk) :: multforbalancedbudg(numtypes)  
       integer :: failbyyear(numyears, 4)
       real (kind=rk) :: failpropbyyear(numyears, 4)
       integer :: ixar1node            

!Individual (type-level) data on earnings, fixed effects and growth rates that will be used in 'optimality' project
    real (kind=rk) :: indearnparams(6, numtypes)
    type(typeopt_realisations) :: allopt_realised(numtypes)
    real (kind=rk) :: unemprates(numtypes)
            

   
    real (kind=rk) :: newannuityrates(numyears, 3)
    logical :: onlyprintingmoments
    real (kind=rk) :: numkidsalltypes(numyears, numtypes), numeqkidsalltypes(numyears, numtypes)
    
    intrinsic :: log



! For finding the discuont rate
            real (kind=rk) :: lowerBoundNewBeta, upperBoundNewBeta, eps, eta, newBeta, estimatedBeta, ruser
            integer ::  ifail, iuser
            ! For printing in finding beta
            character(80) path_in, path_out, out, outFoundBetas
            
! ----------------------------------------------------------------------!
! End of declarations												   !
! ----------------------------------------------------------------------!

            
! Print some summaries about what the program is doing
    !
    call getpaths(path_in, path_out)
    out = trim(path_out) // 'summary.txt'   
    open (unit=450, file = out, action='write', status = 'replace')  


    write(*,*) '!---------------------------------------------------------!'
    write(*,*) '!  PLEASE READ'
    write(*,*) '!---------------------------------------------------------!'
    
    write(*,'(A)', advance = 'no') '-- Solving for the following number of people'
    write(*, '(I5)') numtypes
    write(*,*) 
    write(450,'(A)', advance = 'no') '-- Solving for the following number of people'
    write(450, '(I5)') numtypes
    write(450,*) 
    
    ! First on solving or simulating.
    if (solveForBeta) then
        if (useInputtedBetaAsStartingPoint) then
            write(*,'(A)') trim(adjustl('-- We are estimating USING using previously estimated betas for starting'))
            write(450,'(A)') trim(adjustl('-- We are estimating USING using previously estimated betas for starting'))
        else
            write(*,'(A)') '-- We are estimating WITHTOUT previously estimated betas as a starting point'
            write(450,'(A)') '-- We are estimating WITHTOUT previously estimated betas as a starting point'
        end if
    else
        if (usePreviousEstimatedBeta) then
            write(*,'(A)') '-- We are not estimating. Solving and simulating behaviour using previously estimated betas'
            write(450,'(A)') '-- We are not estimating. Solving and simulating behaviour using previously estimated betas'
        else 
            write(*,'(A)') '-- We are not estimating. Solving and simulating behaviour using beta = 1/1+r'
            write(450,'(A)') '-- We are not estimating. Solving and simulating behaviour using beta = 1/1+r'
        end if
    end if
    write(*,*)
    write(450,*)
        
        
    if (whichModel .eq. 1) then
         write(*,'(A)') '-- Running Model 1. One Asset. Matching all data'
         write(450,'(A)') '-- Running Model 1. One Asset. Matching all data'
    else if (whichModel .eq.2) then
         write(*,'(A)') '-- Running Model 2. Two assets - no modelling of housing. Matching all data'    
         write(450,'(A)') '-- Running Model 2. Two assets - no modelling of housing. Matching all data'    
    else if (whichModel .eq. 3) then
         write(*,'(A)') '-- Running Model 3. Two assets - no modelling of housing. Matching only non-housing data' 
         write(450,'(A)') '-- Running Model 3. Two assets - no modelling of housing. Matching only non-housing data' 
    else if (whichModel .eq. 4) then
         write(*,'(A)') '-- Running Model 4. Two assets - housing model. Matching all data'    
         write(450,'(A)') '-- Running Model 4. Two assets - housing model. Matching all data'    
    end if
    write(*,*)   
    write(450,*)   
        
         
    
    if (homogSurvProbs) then
        write(*,'(A)') '-- Homogenous survival probabilities imposed'
        write(450,'(A)') '-- Homogenous survival probabilities imposed'
    else
        write(*,'(A)') '-- Homogenous survival probabilities NOT imposed'
        write(450,'(A)') '-- Homogenous survival probabilities NOT imposed'
    end if        
    write(*,*)    
    write(450,*)    
    
   if (dbAge.gt.100) then
        write(*,'(A)') '-- DB is Turned Off. NOTE THE CODE DOES NOT ENSURE CORRECT DB-RELEVANT FILES ARE READ IN'
        write(450,'(A)') '-- DB is Turned Off. NOTE THE CODE DOES NOT ENSURE CORRECT DB-RELEVANT FILES ARE READ IN'
   else
        write(*,'(A)', advance = 'no') '-- DB is Turned On.  NOTE THE CODE DOES NOT ENSURE CORRECT DB-RELEVANT FILES ARE READ IN. Eligibility at' 
        write(*, '(I6)') dbAge
        write(450,'(A)', advance = 'no') '-- DB is Turned On.  NOTE THE CODE DOES NOT ENSURE CORRECT DB-RELEVANT FILES ARE READ IN. Eligibility at' 
        write(450, '(I6)') dbAge
   end if
     write(*,'(A)')    
    write(450,'(A)')
       
    if (solveUsingEuler) then
        write(*,'(A)') '-- Solving with Euler equation'
        write(450,'(A)') '-- Solving with Euler equation'
    else
        write(*,'(A)') '-- Solving by maximizing value function'
        write(450,'(A)') '-- Solving by maximizing value function'
    end if
    write(*,*)   
    write(450,*)   
    
    if (optimMethod.eq.1) then
        write(*,'(A)') '-- Solving using local method'
        write(450,'(A)') '-- Solving using local method'
    else if (optimMethod.eq.2) then
        write(*,'(A)') '-- Solving using grid search method'            
        write(450,'(A)') '-- Solving using grid search method'            
    end if
    write(*,*)  
    write(450,*)  


    if (doAveFixedEffect) then
        write(*,'(A)') '-- doAveFixedEffect is On'
        write(450,'(A)') '-- doAveFixedEffect is On'
    else 
        write(*,'(A)') '-- doAveFixedEffect is Off'            
        write(450,'(A)') '-- doAveFixedEffect is Off'            
    end if
    write(*,*)  
    write(450,*)  
    
    if (everyoneHasHousing) then
        write(*,'(A)') '-- Everyone has housing is On'
        write(450,'(A)') '-- Everyone has housing is On'
    else 
        write(*,'(A)') '-- Everyone has housing is Off'            
        write(450,'(A)') '-- Everyone has housing is Off'            
    end if
    write(*,*)  
    write(450,*)  

    
    if (unempOff) then
        write(*,'(A)') '-- unempOff is On'
        write(450,'(A)') '-- unempOff is On'
    else 
        write(*,'(A)') '-- unempOff is Off'            
        write(450,'(A)') '-- unempOff is Off'            
    end if
    write(*,*)  
    write(450,*)  
    
    
    
    if (noPenTaxAdv) then
        write(*,'(A)') '-- noPenTaxAdv is On'
        write(450,'(A)') '-- noPenTaxAdv is On'
    else 
        write(*,'(A)') '-- noPenTaxAdv is Off'            
        write(450,'(A)') '-- noPenTaxAdv is Off'            
    end if
    write(*,*)  
    write(450,*)  
    
    
    write(*,*)  
    write(450,*)     
    
    close(450)

    
    
    

    
    
            
! ----------------------------------------------------------------------!
! If we're going to be solving for beta, delete files from previous run
! ----------------------------------------------------------------------!
    
    call getpaths(path_in, path_out)
    out = trim(path_out) // 'findingBeta.txt'   
    open (unit=40, file = out, action='write', status = 'replace')  
    close(40)
    outFoundBetas = trim(path_out) // 'foundBetas.txt'     
    open (unit=41, file = outFoundBetas, action='write', status = 'replace') 
    close(41)
    
    

     call cpu_time(time1)

     
! Which model are we in
    if (whichModel.eq.1) then
        dcOverRide = .true.
        intRateOverRide = .true.
        intMeth = 1             
        dcOverrideVal = -0.75_rk ! Redundant
    else if ((whichModel.eq.2).or.(whichModel.eq.3)) then
        dcOverRide = .false.
        intRateOverRide = .true.
        intMeth = 1       
    else if (whichModel.eq.4) then
        intRateOverRide = .false.
        dcOverRide = .false.
    else if (whichModel.eq.5) then
        dcOverRide = .true.
        intRateOverRide = .false.    
    end if
    

     
!-------------------------------------------------------------------------!
! Read in the inputs
!-------------------------------------------------------------------------!
 
      call readinpoints(allpoints, numkidsalltypes, numeqkidsalltypes, indearnparams, allopt_realised, unemprates)
!-------------------------------------------------------------------------!
      callingFromVFI = .false.
      firstcall = .true.



do ixtype = 1, numtypes, 1

        cashData = allopt_realised(ixtype)%cashData
        penData= allopt_realised(ixtype)%penData
        housingData = allopt_realised(ixtype)%housData
        nonpenData = allopt_realised(ixtype)%npData
        
        ageData = 2002 - allopt_realised(ixtype)%yob 
  
        call cpu_time(timetypestart)
        
        points = allpoints(ixtype) !This must be above the following allocations                                  

        ! This is for in linearinterp2_fast - to speed up.
        log2 = log(real(2,rk))
        numCycles1 = log(real(points%ass+1,rk))/(log2)  + 1
        numCycles2 = log(real(points%dc+1,rk))/(log2)  + 1

        call get3waysurvivalprob(points%numyears, points%survprobs_m, points%survprobs_f, points%usesurvprob, points%survprobs3way)
        

        doprintmoments = .false.
        allocate (points%fairannuity(numyears, 3))

        ! Get the grid of consumption
        allocate(consGrid(points%Cons))
        call getnodes(consGrid, 0.0_rk, 1.0_rk, points%Cons, 0.0_rk)
        
        !----------------------------!
        !Allocations
        !----------------------------!
        allocate (exval%pre(points%ass,points%dc,points%ar1by4,points%numhhcomp,claimAge-19))
        allocate (exval%post(points%ass,points%pen,points%ar1,points%housPoints, points%numhhcomp,(claimAge-19+1):(points%numyears+1)))
        allocate (exval%mu_pre(points%ass,points%dc,points%ar1by4,points%numhhcomp,claimAge-19))
        allocate (exval%mu_post(points%ass,points%pen,points%ar1,points%housPoints, points%numhhcomp,(claimAge-19+1):(points%numyears+1)))
        allocate (exval%emu_pre(points%ass,points%dc,points%ar1by4,points%numhhcomp,claimAge-19))
        allocate (exval%emu_post(points%ass,points%pen,points%ar1,points%housPoints, points%numhhcomp,(claimAge-19+1):(points%numyears+1)))

        allocate (policy%c_pre(points%ass,points%dc,points%ar1by4,points%numhhcomp,claimAge-19)) !policy%c_pre
        allocate (policy%c_post(points%ass,points%pen,points%ar1,points%housPoints,points%numhhcomp,(claimAge-19+1):points%numyears)) !policy%c_post
           
        allocate (policy%dc_pre(points%ass,points%dc,points%ar1by4,points%numhhcomp,claimAge-19)) !policy%dc_pre
                
        allocate (ar1shocks(points%ar1by4,points%numyears))
        allocate (ar1pmat(points%ar1by4,points%ar1by4,points%numyears))
        allocate (ar1cdf(points%ar1by4,points%ar1by4,points%numyears))
        allocate (allearnings(workChoices,points%ar1by4,points%numyears))
        allocate (points%dcquad%weigh(points%dcquad%dim))
        allocate (points%dcquad%abs(points%dcquad%dim))
        allocate (assgrid(points%ass,points%numyears+1))
        allocate (housgrid(points%housPoints,  (65-19):points%numyears))
        allocate (dcgrid(points%dc,claimAge-19))
        allocate (pengrid(points%pen))
        allocate (ar1shocksnoue2(points%ar1,points%numyears))
        allocate (ar1pmatnoue2(points%ar1,points%ar1,points%numyears))
        allocate (ar1cdfnoue2(points%ar1,points%ar1,points%numyears))
        allocate (allcpaths(points%numyears, points%sims))
        allocate (alllpaths(points%numyears, points%sims))
        allocate (allhhcomps(points%numyears, points%sims))
        allocate (points%popEarQuantsAt64(points%ar1))
        
           !Get the fair annuity rates        
        call getannuityratesnew(points%numyears,points%survprobs_m,points%survprobs_f,points%rate,points%fairannuity, &
                                                         points%penpropinherit, points%annAdminLoad) 
        

        
        call printannuityrates(points)
        
        call getdcquadweights(points, points%dcquad%weigh, points%dcquad%abs)

! First get population level earnings process        
        call getprocesses(numyears, points%earnPop, points%ar1, 64, detincome, ar1shocksnoue2, ar1pmatnoue2, ar1cdfnoue2)
        do ixar1node = 1, points%ar1
            points%popEarQuantsAt64(ixar1node) = detincome(64-19)*ar1shocksnoue2(ixar1node, 64-19)
        end do
        
        
! Get income process
        ! If everyone has the average fixed effect then we replace the constant here with the population level earnings 
        if (doAveFixedEffect) then
            points%earn%params(1) = points%earnPop%params(1)
        end if
        

        call getprocesses(numyears, points%earn, points%ar1, 64, detincome, ar1shocksnoue2, ar1pmatnoue2, ar1cdfnoue2)                
        call suppressfanningoutofincome( points%ar1, numyears, detincome, ar1shocksnoue2, ar1pmatnoue2, ar1cdfnoue2, lastWorkAge)

        call adduneandwomentoincome(points, ar1shocksnoue2, ar1pmatnoue2, ar1cdfnoue2, ar1shocks,ar1pmat, ar1cdf)
        
! Generate the  asset grid, dc grid and pension grid
        call getgrids(points, detincome, ar1shocksnoue2, points%rate, assgrid, dcgrid, pengrid, housgrid)

!Get an array at each work choice at each shock holding all the earnings 
        call getearningsarry(points, detincome, ar1shocks, allearnings)

! Convert realised earnings into discretised earnigns
              call convertcontinuous_to_discrete(points, allearnings, allopt_realised(ixtype))
              doprintsimdetail = .false.
    
            !! Either estimate beta or solve the model for a given beta
            if (solveForBeta) then
                
                if (useInputtedBetaAsStartingPoint) then
                
                    
                    lowerBoundNewBeta = points%params%beta - 0.01_rk
                    upperBoundNewBeta = points%params%beta + 0.01_rk
                    eps = 0.0000005
                    eta = 1000_rk
                    ifail = 1
              
                   ! For NAG MARK 26 - OUT  NAG MARK 24 - IN
                    !call c05adf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRate, estimatedBeta, ifail)
                
                   ! For NAG MARK 26 - IN  NAG MARK 24 - OUT
                    call c05ayf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRateNAG26,estimatedBeta, iuser, ruser, ifail)
                
                    if (ifail.ne.0) then ! do a broader search
                        lowerBoundNewBeta = 0.5_rk
                        upperBoundNewBeta = 1.5_rk
                        eps = 0.0000005
                        eta = 1000_rk
                        ifail = 1
              
                       ! For NAG MARK 26 - OUT  NAG MARK 24 - IN
                        !call c05adf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRate, estimatedBeta, ifail)
                
                       ! For NAG MARK 26 - IN  NAG MARK 24 - OUT
                        call c05ayf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRateNAG26,estimatedBeta, iuser, ruser, ifail)
                    end if
                    
                else 
                    lowerBoundNewBeta = 0.5_rk
                    upperBoundNewBeta = 1.5_rk
                    eps = 0.0000005
                    eta = 1000_rk
                    ifail = 1
              
                   ! For NAG MARK 26 - OUT  NAG MARK 24 - IN
                    !call c05adf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRate, estimatedBeta, ifail)
                
                   ! For NAG MARK 26 - IN  NAG MARK 24 - OUT
                    call c05ayf(lowerBoundNewBeta, upperBoundNewBeta, eps, eta, diffForDiscRateNAG26,estimatedBeta, iuser, ruser, ifail)
               end if     
                
                
                
                
                    write(*,*)
                    write(*,*) ixtype, estimatedBeta, ifail                
                    
                    ! Print it in the file with all the runs
                    open (unit=40, file = out, action='write', position = 'append')
                    write(40,*) ixtype, estimatedBeta, ifail                                    
                    write(40,*)
                    close(40)
                    
                    !And now print it in the file with just the results
                    open (unit=41, file = outFoundBetas, action='write', position = 'append')
                    write(41,*) ixtype, estimatedBeta, ifail                                    
                    write(*,*)
                    close(41)
                    
                    
                    
            else
                call solveandsim
                call getreportcard(numyears,firstcall, ixtype, sdffail, sdffailprop,failbyyear,failpropbyyear, failmaxoversims, failaveoversims)
                    ! Print it in the file with all the runs
                    open (unit=40, file = out, action='write', position = 'append')
                    write(40,*) ixtype       

                     !And print out beta
                    open (unit=41, file = outFoundBetas, action='write', position = 'append')
                    write(41,*) ixtype, points%params%beta, ifail                                    
                    write(*,*)
                    close(41)
                 
            end if
            

    
       
        call dodeallocate
        
        call cpu_time(timetypeend)
        timealltypes(ixtype) = timetypeend - timetypestart
        firstcall = .false.


end do ! ixtype
  
        call printstats(alltypstats)

             
      !Print details on the different types

       do ixtype = 1, numtypes, 1
       write (*, *) 'Time taken for type', ixtype, 'was', timealltypes(ixtype), 'seconds'
       end do



      call cpu_time(time2)
      write (*, *) 'Time taken by code was', time2 - time1, 'seconds'

    contains
    
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
real (kind=rk) function diffForDiscRateNAG26(newBeta, iuser, ruser)
    real (kind=rk) :: newBeta, ruser
    integer :: iuser
    
    diffForDiscRateNAG26 =  diffForDiscRate(newBeta)
end function diffForDiscRateNAG26

!---------------------------------------!
!Find difference between asset in data and asset in mode
!---------------------------------------!

real (kind=rk) function diffForDiscRate(newBeta)
    real (kind=rk) :: newBeta
    
    ! For programme
    real (kind=rk) :: dataForMatch, simForMatch
    
    ! For printing
    character (80) :: path_in, path_out, out
    
    call getpaths(path_in, path_out)

    out = trim(path_out) // 'findingBeta.txt' 
    
    open (unit=40, file = out, action='write', position = 'append')
    
    points%params%beta = newBeta

    call solveandsim
    
    if (whichModel.eq.3) then
        dataForMatch = cashData + penData
        simForMatch = cashSim + penSim 
        if (housingSim.gt.0.0000001_rk) then
            write(*,*) 'Housing should be zero in model 3'
            stop
        end if       
    else
        dataForMatch = cashData + penData + housingData
        simForMatch = cashSim + penSim +housingSim
    end if
    
    diffForDiscRate =  dataForMatch - simForMatch
    
    
    ! Print running commmentary to screen and to file
    write(*,'(1f15.6)', ADVANCE = 'no') newBeta
    write(*,'(6f15.1)') diffForDiscRate, dataForMatch, simForMatch, cashSim, penSim, housingSim
    write(40,'(4f15.6)') newBeta, diffForDiscRate, dataForMatch,  simForMatch  
    close(40) ! until next time
    
    
    
end function diffForDiscRate

!----------------------------!
!THE ENGINE ROOM - Solve and simulate
!----------------------------!

subroutine solveandsim
  implicit none
        
        call vfi(points, allearnings,ar1pmat, assgrid, dcgrid, pengrid, housgrid,allopt_realised(ixtype), policy, exval, sdffail, &
          sdffailprop,failbyyear,failpropbyyear)
        
        
        call simulate(points, ixtype, points%sims, allearnings, ar1shocks, ar1pmat, ar1cdf, doprintsimdetail, assgrid, &
            dcgrid, pengrid, housgrid, policy, exval, allopt_realised(ixtype),failmaxoversims, failaveoversims, alltypstats(ixtype), &
            expdiscutil(ixtype),allcpaths,alllpaths,allhhcomps)


end subroutine solveandsim


! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!



! ---------------------------------------------------------------------------------------------------------!
subroutine dodeallocate
  implicit none
        deallocate (exval%pre)
        deallocate (exval%post)
        deallocate (exval%mu_pre)
        deallocate (exval%mu_post)
        deallocate (exval%emu_pre)
        deallocate (exval%emu_post)
       
        deallocate (policy%c_pre)
        deallocate (policy%c_post)
        deallocate (policy%dc_pre)
        deallocate (points%dcquad%weigh)
        deallocate (points%dcquad%abs)
        deallocate (ar1shocks)
        deallocate (ar1pmat)
        deallocate (ar1cdf)
        deallocate (allearnings)
        deallocate (assgrid)
        deallocate (dcgrid)
        deallocate (pengrid)
        deallocate (housgrid)
        deallocate (ar1shocksnoue2)
        deallocate (ar1pmatnoue2)
        deallocate (ar1cdfnoue2)            
        deallocate (allcpaths)
        deallocate (alllpaths)
        deallocate (allhhcomps)
        deallocate (points%fairannuity)       
        deallocate (consGrid)
        deallocate (points%popEarQuantsAt64)
        
  end subroutine dodeallocate
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
    end program main
