
    module pension
      use precisions
      use types_generic
      use globalvalues
    contains
!--------------------------------------------------------------------------!
!--------------------------------------------------------------------------!
        real (kind=rk) function guarcred(pc,todaynetinclessdc, assets, todayage, hhcomp)

        implicit none

!Arguments
        type (pcsys), intent (in) :: pc
        real (kind=rk), intent (in) :: todaynetinclessdc
        real (kind=rk), intent (in) :: assets
        integer, intent (in) :: todayage
        integer, intent (in) :: hhcomp
        
!For programme
        real (kind=rk) :: incomefortest
        real (kind=rk) :: incfromcap
        
        if (todaynetinclessdc.lt.0_rk) then
           write(*,*) 'error in guarcred, income is negative'
           stop
        end if
        
        
        
        if ((todayage.lt.pc%eligage).or.(.not.(pc%guarcredon))) then
            guarcred = 0
        else
            incfromcap = (max(assets - pc%capdisreg, 0.0_rk))*pc%capflowrate
            incomefortest = todaynetinclessdc + incfromcap
        
            if (hhcomp.eq.1) then !couples
                guarcred = max(pc%levelcoup - incomefortest, 0.0_rk)
            else 
                guarcred = max(pc%level - incomefortest, 0.0_rk)
            end if
        end if                                            
        
      
      end function guarcred
!--------------------------------------------------------------------------!
!--------------------------------------------------------------------------!
       real (kind=rk) function savcred(pc,todaynetinclessdc, assets, todayage, hhcomp)

        implicit none

!Arguments
        type (pcsys), intent (in) :: pc
        real (kind=rk), intent (in) :: todaynetinclessdc
        real (kind=rk), intent (in) :: assets
        integer, intent (in) :: todayage
        integer, intent (in) :: hhcomp
        
!For programme
        real (kind=rk) :: incomefortest
        real (kind=rk) :: incfromcap
        
        if (todaynetinclessdc.lt.0_rk) then
           write(*,*) 'error in savcred, income is negative'
           stop
        end if
        
        
        incfromcap = (max(assets - pc%capdisreg, 0.0_rk))*pc%capflowrate
        incomefortest = todaynetinclessdc + incfromcap
        
        if (hhcomp.eq.1) then !couples
            if ((todayage.lt.pc%sceligage).or.(.not.(pc%savcredon)).or.(incomefortest.gt.pc%maxincforsavcredcoup)&
            .or.(incomefortest.lt.pc%scthresholdcoup)) then
                savcred = 0
            else if (incomefortest.lt.pc%levelcoup) then
                savcred = max((incomefortest - pc%scthresholdcoup) * (pc%oneminustaper),0.0_rk)
            else if ((incomefortest.gt.pc%levelcoup).and.(incomefortest.lt.pc%maxincforsavcredcoup)) then
                savcred = pc%levelcoup - pc%scthresholdcoup -pc%taper*(incomefortest - pc%scthresholdcoup)                              
            else
                write(*,*) 'error in savcred'
                stop
            end if
        
        else    !singles
            if ((todayage.lt.pc%sceligage).or.(.not.(pc%savcredon)).or.(incomefortest.gt.pc%maxincforsavcred)&
            .or.(incomefortest.lt.pc%scthreshold)) then
                savcred = 0
            else if (incomefortest.lt.pc%level) then
                savcred = max((incomefortest - pc%scthreshold) * (pc%oneminustaper),0.0_rk)
            else if ((incomefortest.gt.pc%level).and.(incomefortest.lt.pc%maxincforsavcred)) then
                savcred = pc%level - pc%scthreshold -pc%taper*(incomefortest - pc%scthreshold)                              
            else
                write(*,*) 'error in savcred'
                stop
            end if
        end if
                  
        if (savcred.lt.0) then
            write(*,*) ' savings credit is less than zero'
            stop
        end if   
         
     
      end function savcred
!--------------------------------------------------------------------------!
!--------------------------------------------------------------------------!

      real (kind=rk) function pension_dc(points, annuityrate, retage, dcpot)
        implicit none

!Arguments
        type (grids), intent (in) :: points
        real (kind=rk), intent (in) :: dcpot
        real (kind=rk), intent(in) :: annuityrate
        integer, intent (in) :: retage

          pension_dc = (dcpot*annuityrate)
      end function pension_dc
!--------------------------------------------------------------------------!
!--------------------------------------------------------------------------!

      real (kind=rk) function statepen(points, age,opt_realised, decileAt64, todayinfo)
        implicit none
        type (grids), intent (in) :: points
        integer, intent (in) :: age
        real (kind=rk), intent(in) :: decileAt64
        type(typeopt_realisations), intent(in)  :: opt_realised
        type (typetodayinfo), intent(in) :: todayinfo

        
        
            if (age.ge.points%statepenage) then      

                if ((todayinfo%shockDecile.lt.0.0_rk).or.(todayinfo%shockDecile.gt.10.0_rk)) then
                        write(*,*) 'error in statepen, shockDecile should be between zero and 1'            
                        stop
                end if         
                
                statepen = points%statepencoeffs(1) &
                    + (points%statepencoeffs(2)   * points%FEdecile) +     &
                    + (points%statepencoeffs(3)   * (points%FEdecile **2)) +      &
                    + (points%statepencoeffs(4)   * decileAt64) +      &
                    + (points%statepencoeffs(5)   * (decileAt64 ** 2)) +      &
                    + (points%statepencoeffs(6)   * (points%FEdecile * decileAt64 ))                
               if (todayinfo%ixhhcomp .ne.1) then
                 statepen = statepen * 0.5_rk
               end if
            else
              statepen = 0.0_rk
            end if


      end function statepen
      
!-----------------------------------------------------------------------------------------------------------!
!-----------------------------------------------------------------------------------------------------------!
      
      real (kind=rk) function db(points,age,opt_realised, decileAt64, todayinfo)
        implicit none
        type (grids), intent (in) :: points
        integer, intent (in) :: age
        type(typeopt_realisations), intent(in)  :: opt_realised
        type (typetodayinfo), intent(in) :: todayinfo
        real (kind=rk), intent(in) :: decileAt64



        
        if ((age.ge.dbAge).and.(points%hasDB)) then
            if ((todayinfo%shockDecile.lt.0.0_rk).or.(todayinfo%shockDecile.gt.10.0_rk)) then
                    write(*,*) 'error in db, shockDecile should be between zero and 1'            
                    stop
            end if          
            db = points%dbcoeffs(1) &
                    + (points%dbcoeffs(2)   * points%FEdecile) +     &
                    + (points%dbcoeffs(3)   * (points%FEdecile **2)) +      &
                    + (points%dbcoeffs(4)   * decileAt64) +      &
                    + (points%dbcoeffs(5)   * (decileAt64 ** 2)) +      &
                    + (points%dbcoeffs(6)   * (points%FEdecile * decileAt64 ))
            
            if (todayinfo%ixhhcomp.gt.1) db = db/2.0_rk                  
        else 
           db = 0.0_rk
        end if 


            if (db.lt.0.0_rk)  db = 0.0_rk
            
      end function db
!-----------------------------------------------------------------------------------------------------------!
!-----------------------------------------------------------------------------------------------------------!

!!--------------------------------------------------------------------------!
!!--------------------------------------------------------------------------!
      subroutine getsaveamounts(points, earnings, dcchoices, todaydcsave_array)
        implicit none

!Arguments
        type (grids), intent (in) :: points
        real (kind=rk), intent (in) :: earnings(workChoices)
        real (kind=rk), intent (in) :: dcchoices(points%numdcch)
        real (kind=rk), intent (out) :: todaydcsave_array(workChoices, points%numdcch)

!For programm
        integer ixlab, ixdcchoice

        do ixlab = 1, workChoices, 1
          do ixdcchoice = 1, points%numdcch, 1
            todaydcsave_array(ixlab, ixdcchoice) = earnings(ixlab)*dcchoices(ixdcchoice)
          end do !ixdcchoice
        end do !ixlab


      end subroutine getsaveamounts


    end module pension
