
! This contains generic routines
! Random number routines here are scheduled for withdrawal. Need to replace with new ones
    module routines_generic
      use precisions

    contains
      subroutine linearinterp1(xgrid, ygrid, length, xval, yval, extrapbot, extraptop)
! Subroutine for linear interpolation
! Arguments are 1. (input) xvlalues
! 2. (input) yvalues
! 3. (input) length of xgrid and ygrid,
! 4. (input) value of x to interpolate at
! 5. (output) interpolated value
! 6. (input) indicator for whether to extrapolate below bottom point of grid
! 7. (input indicator for whether to extrapolate above top point of grid

! If the choose not to extrapolate interpolated value is set equal to either the top or the bottom
! point of the grid
! It is fully generic

        implicit none
! Arguments
        integer, intent (in) :: length
        integer, intent (in) :: extrapbot
        integer, intent (in) :: extraptop
        real (kind=rk), intent (in) :: xgrid(length)
        real (kind=rk), intent (in) :: ygrid(length)
        real (kind=rk), intent (in) :: xval
        real (kind=rk), intent (out) :: yval

! For programme
        integer :: xlowerloc
        integer scratch_xnearestloc ! This areguments required by findingridr but I'm not using them here 
! scratch
        integer :: jl, ju, jm
        intrinsic size

! Checking inputs - have commented this out for speed
        if ((size(xgrid).ne.length) .or. (size(ygrid).ne.length)) then
          write (*, *) 'An argument in linearinterp1 is not of the specified length'
          stop
        end if

        if ((extrapbot.ne.0 .and. extrapbot.ne.1) .or. (extraptop.ne.0 .and. extraptop.ne.1)) then
          write (*, *) 'An argument in linearterp1 that needs to be either 0 or 1 is not so'
          stop
        end if

        if (abs(xgrid(length)-xgrid(1)).lt.0.1d-15) then ! If the top of the grid is equal to the bottom
          yval = ygrid(1) ! (which would include the scenario that all the entries are the same) then I
! assign the bottom value of y to yval
        else
      
        ! Find the bounds    
        jl = 0
        ju = length + 1
        do while (ju-jl.gt.1)
          jm = (ju+jl)/2
          if (xval.gt.xgrid(jm)) then
            jl = jm
          else
            ju = jm
          end if
        end do

        xlowerloc = jl
            
          call linearinterpfromlocations(length, xgrid, ygrid, xlowerloc, xval,  extrapbot, extraptop, yval)
        end if

      end subroutine linearinterp1

! ---------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!


! Extrapolation is by default here. There is no option to use the bottom value (as I have
! in the one-dimensional linear interpolation).

      subroutine linearinterp2_withextrap(x1grid, x2grid, dim1, dim2, x1val, x2val, yval, extrapbot, extraptop, ygrid)
        implicit none
! Arguments
        integer, intent (in) :: dim1
        integer, intent (in) :: dim2
        real (kind=rk), intent (in) :: x1grid(dim1)
        real (kind=rk), intent (in) :: x2grid(dim2)
        real (kind=rk), intent (in) :: ygrid(dim1, dim2)
        real (kind=rk), intent (in) :: x1val
        real (kind=rk), intent (in) :: x2val
        real (kind=rk), intent (out) :: yval
        integer, intent (in) :: extrapbot
        integer, intent (in) :: extraptop

! For programme
        integer :: x1lowerloc
        integer :: x2lowerloc
        integer :: scratch_nearestloc
        integer :: newxgridlowerloc
        real (kind=rk) :: newygrid(2)
        real (kind=rk) :: newxgrid(2)

        if ((size(x1grid).ne.dim1) .or. (size(x2grid).ne.dim2)) then
          write (*, *) 'An argument in linearinterp2 is not of the specified length'
          stop
        end if

! Find the square
        call findingridr(x1grid, dim1, x1val, scratch_nearestloc, x1lowerloc)
        call findingridr(x2grid, dim2, x2val, scratch_nearestloc, x2lowerloc)

! Find if any of the values are off the grid
        if (x1lowerloc.eq.0) then
          x1lowerloc = 1
        end if

        if (x2lowerloc.eq.0) then
          x2lowerloc = 1
        end if

        if (x1lowerloc.eq.dim1) then
          x1lowerloc = dim1 - 1
        end if

        if (x2lowerloc.eq.dim2) then
          x2lowerloc = dim2 - 1
        end if

! Now do linearinterpolation along the first dimension
        call linearinterpfromlocations(dim1, x1grid, ygrid(:,x2lowerloc), x1lowerloc, x1val,  extrapbot, extraptop, &
          newygrid(1))
        call linearinterpfromlocations(dim1, x1grid, ygrid(:,(x2lowerloc+1)), x1lowerloc, x1val,  extrapbot, extraptop, &
          newygrid(2))

! Now generate a new grid from from the x2 bounds (a new ygrid has already bee generated)
        newxgrid(1) = x2grid(x2lowerloc)
        newxgrid(2) = x2grid(x2lowerloc+1)

        if (x2lowerloc.eq.0) then
          newxgridlowerloc = 0
        else if (x2lowerloc.eq.dim2) then
          newxgridlowerloc = 2
        else
          newxgridlowerloc = 1
        end if
        call linearinterpfromlocations(2,newxgrid, newygrid, newxgridlowerloc, x2val,  extrapbot, extraptop, yval)

      end subroutine linearinterp2_withextrap
!
! ---------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!
! Extrapolation is by default here. There is no option to use the bottom value (as I have
! in the one-dimensional linear interpolation).

      !linearinterp2_fast has the following difference from linearinterp2
      ! 1. It doesn't call findingr but does it within this module
      ! 2. It checks if the last saved value for the lowerlocs are still correct and if so, doesn't do a binary search
      
      subroutine linearinterp2_fast(x1grid, x2grid, dim1, dim2, x1val, x2val, yval, ygrid)
        use types
        implicit none
! Arguments
        integer, intent (in) :: dim1
        integer, intent (in) :: dim2
        real (kind=rk), intent (in) :: x1grid(dim1)
        real (kind=rk), intent (in) :: x2grid(dim2)
        real (kind=rk), intent (in) :: x1val
        real (kind=rk), intent (in) :: x2val
        real (kind=rk), intent (out) :: yval
        real (kind=rk), intent (in) :: ygrid(dim1, dim2)
        
        
! For programme
        integer :: x1lowerloc, x2lowerloc
        integer, save :: x1lowerloc_save, x2lowerloc_save
        integer :: scratch_nearestloc
        integer :: newxgridlowerloc
        real (kind=rk) :: newygrid(2)
        real (kind=rk) :: newxgrid(2)
         
        integer ::  x1upperloc
         real (kind=rk) :: xgridmin, xgridmax, xgriddiffinv, weightmax, weightmin
                integer :: jl, ju, jm
       logical ::        binarySearch
integer :: tempCounter, numCycles
        
        !Have commented out for speed
        !if ((size(x1grid).ne.dim1) .or. (size(x2grid).ne.dim2)) then
        !  write (*, *) 'An argument in linearinterp2 is not of the specified length'
        !  stop
        !end if

    !!!___________________________________!!!!
    !!! Find Square
    !!!___________________________________!!!!
                
                
    ! Do find in grid without calling the routine....for speed...calling the routine slows us down
      ! This block of code finds the two points on the xgrid between which the x value lies        
        !call findingridr(x1grid, dim1, x1val, scratch_nearestloc, x1lowerloc)

       binarySearch = .true.
        
       if ((x1lowerloc_save.lt.1).or.(x1lowerloc_save.gt.(dim1 - 1))) then
            binarySearch = .true.
       else if (.not.((x1val.gt.x1grid(x1lowerloc_save)).and.(x1val.lt.x1grid(x1lowerloc_save+1)))) then
            binarySearch = .true.
       else
            binarySearch = .false.
       end if
       
                   
       if (binarySearch) then
                    jl = 0
                    ju = dim1 + 1
                    !do  tempCounter = 1, numCycles1
                    do while (ju-jl.gt.1)
                   
                      jm = (ju+jl)/2
                      if (x1val.gt.x1grid(jm)) then
                        jl = jm
                      else
                        ju = jm
                      end if
                    end do
                   !write(*,*) tempCounter
                    
                    
                    x1lowerloc = jl

                if (x1lowerloc.eq.0) then
                  x1lowerloc = 1
                end if
                if (x1lowerloc.eq.dim1) then
                  x1lowerloc = dim1 - 1
                end if

        else  ! The bounds from last call are still the correct ones      
                
                 x1lowerloc = x1lowerloc_save            
        end if
        

        

        ! Save for next run
        x1lowerloc_save = x1lowerloc


       binarySearch = .true.
        
       if ((x2lowerloc_save.lt.1).or.(x2lowerloc_save.gt.(dim2 - 1))) then
            binarySearch = .true.
       else if (.not.((x2val.gt.x2grid(x2lowerloc_save)).and.(x2val.lt.x2grid(x2lowerloc_save+1)))) then
            binarySearch = .true.
       else
            binarySearch = .false.
       end if
        
        
      
       if (binarySearch) then

            jl = 0
            ju = dim2 + 1
          
            do while (ju-jl.gt.1)
            !do  tempCounter = 1, numCycles2
              jm = (ju+jl)/2
              if (x2val.gt.x2grid(jm)) then
                jl = jm
              else
                ju = jm
              end if
            end do

            x2lowerloc = jl

            ! Find if any of the values are off the grid
        
              if (x2lowerloc.eq.0) then
                x2lowerloc = 1
              end if
              if (x2lowerloc.eq.dim2) then
                x2lowerloc = dim2 - 1
              end if
        else        
                    ! The bounds from last call are still the correct ones
                    x2lowerloc = x2lowerloc_save
        end if        
        
       

          ! Save for next run
          x2lowerloc_save = x2lowerloc


 !!! Do this but faster         
        !call linearinterpfromlocations(dim1, x1grid, ygrid(:,x2lowerloc), x1lowerloc, x1val,  1, 1, newygrid(1))
         
       if (x1lowerloc.ne.0 .and. x1lowerloc.ne.dim1) then
          x1upperloc = x1lowerloc + 1
          xgridmin = x1grid(x1lowerloc)
          xgridmax = x1grid(x1upperloc)
          xgriddiffinv = 1/(xgridmax-xgridmin)
          weightmax = (x1val-xgridmin)*xgriddiffinv
          weightmin = (xgridmax-x1val)*xgriddiffinv
          
          ! Calculating the interpolated function values
          newygrid(1) = weightmin*ygrid(x1lowerloc, x2lowerloc) + weightmax*ygrid(x1upperloc,x2lowerloc)
       else
           call linearinterpfromlocations(dim1, x1grid, ygrid(:,x2lowerloc), x1lowerloc, x1val,  1, 1, newygrid(1))
       end if      
          
          
          
  !!! Do this but faster        
!          call linearinterpfromlocations(dim1, x1grid,ygrid(:,x2lowerloc+1), x1lowerloc, x1val,  1, 1, newygrid(2))

       if (x1lowerloc.ne.0 .and. x1lowerloc.ne.dim1) then
          x1upperloc = x1lowerloc + 1
          xgridmin = x1grid(x1lowerloc)
          xgridmax = x1grid(x1upperloc)
          xgriddiffinv = 1/(xgridmax-xgridmin)
          weightmax = (x1val-xgridmin)*xgriddiffinv
          weightmin = (xgridmax-x1val)*xgriddiffinv
          
          ! Calculating the interpolated function values
          newygrid(2) = weightmin*ygrid(x1lowerloc, x2lowerloc + 1) + weightmax*ygrid(x1upperloc,x2lowerloc + 1)
       else
           call linearinterpfromlocations(dim1, x1grid, ygrid(:,x2lowerloc), x1lowerloc, x1val,  1, 1, newygrid(2))
       end if      
          
          
          
          
! Now generate a new grid from from the x2 bounds (a new ygrid has already bee generated)
        newxgrid(1) = x2grid(x2lowerloc)
        newxgrid(2) = x2grid(x2lowerloc+1)

!Think this is redundant
        if (x2lowerloc.eq.0) then
          newxgridlowerloc = 0
        else if (x2lowerloc.eq.dim2) then
          newxgridlowerloc = 2
        else
          newxgridlowerloc = 1
        end if
        call linearinterpfromlocations(2, newxgrid, newygrid, newxgridlowerloc, x2val,  1, 1, yval)

      end subroutine linearinterp2_fast


     
! ---------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!
! Extrapolation is by default here. There is no option to use the bottom value (as I have
! in the one-dimensional linear interpolation).

      subroutine linearinterp2(x1grid, x2grid, dim1, dim2, x1val, x2val, yval, ygrid)
        use types
        implicit none
! Arguments
        integer, intent (in) :: dim1
        integer, intent (in) :: dim2
        real (kind=rk), intent (in) :: x1grid(dim1)
        real (kind=rk), intent (in) :: x2grid(dim2)
        real (kind=rk), intent (in) :: x1val
        real (kind=rk), intent (in) :: x2val
        real (kind=rk), intent (out) :: yval
        real (kind=rk), intent (in) :: ygrid(dim1, dim2)
        
! For programme
        integer :: x1lowerloc
        integer :: x2lowerloc
        integer :: scratch_nearestloc
        integer :: newxgridlowerloc
        real (kind=rk) :: newygrid(2)
        real (kind=rk) :: newxgrid(2)
        
        !Have commented out for speed
        if ((size(x1grid).ne.dim1) .or. (size(x2grid).ne.dim2)) then
          write (*, *) 'An argument in linearinterp2 is not of the specified length'
          stop
        end if

        call findingridr(x1grid, dim1, x1val, scratch_nearestloc, x1lowerloc)

        if (x1lowerloc.eq.0) then
          x1lowerloc = 1
        end if
        if (x1lowerloc.eq.dim1) then
          x1lowerloc = dim1 - 1
        end if

! Find the square
          call findingridr(x2grid, dim2, x2val, scratch_nearestloc, x2lowerloc)

! Find if any of the values are off the grid
        
          if (x2lowerloc.eq.0) then
            x2lowerloc = 1
          end if
          if (x2lowerloc.eq.dim2) then
            x2lowerloc = dim2 - 1
          end if



          call linearinterpfromlocations(dim1, x1grid, ygrid(:,x2lowerloc), x1lowerloc, x1val,  1, 1, newygrid(1))
          call linearinterpfromlocations(dim1, x1grid, ygrid(:,(x2lowerloc+1)), x1lowerloc, x1val,  1, 1, newygrid(2))

! Now generate a new grid from from the x2 bounds (a new ygrid has already bee generated)
        newxgrid(1) = x2grid(x2lowerloc)
        newxgrid(2) = x2grid(x2lowerloc+1)

!Think this is redundant
        if (x2lowerloc.eq.0) then
          newxgridlowerloc = 0
        else if (x2lowerloc.eq.dim2) then
          newxgridlowerloc = 2
        else
          newxgridlowerloc = 1
        end if
        call linearinterpfromlocations(2, newxgrid, newygrid, newxgridlowerloc, x2val,  1, 1, yval)

      end subroutine linearinterp2


! ---------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!

! Subroutine for interpolating in one dimension given the points on the grid that bound the value for
! evaluation
      subroutine linearinterpfromlocations(length, xgrid, ygrid, xlowerloc, xval, extrapbot, extraptop, yval)
        implicit none
        integer, intent (in) :: length
        real (kind=rk), intent (in) :: xgrid(length)
        real (kind=rk), intent (in) :: ygrid(length)
        integer, intent (in) :: xlowerloc
        real (kind=rk), intent (in) :: xval
        integer, intent (in) :: extrapbot
        integer, intent (in) :: extraptop
        real (kind=rk), intent (out) :: yval


! For programme
        real (kind=rk) xgridmin
        real (kind=rk) xgridmax
        real (kind=rk) xgriddiffinv
        real (kind=rk) weightmin
        real (kind=rk) weightmax
        real (kind=rk) extrapslope
        integer :: xupperloc

! First, and easiest, get the interpolated value if xval is interior to the grid
        if (xlowerloc.ne.0 .and. xlowerloc.ne.length) then
          xupperloc = xlowerloc + 1
          xgridmin = xgrid(xlowerloc)
          xgridmax = xgrid(xupperloc)
          xgriddiffinv = 1/(xgridmax-xgridmin)

          weightmax = (xval-xgridmin)*xgriddiffinv
          weightmin = (xgridmax-xval)*xgriddiffinv

! Calculating the interpolated function values
          yval = weightmin*ygrid(xlowerloc) + weightmax*ygrid(xupperloc)
        else if (xlowerloc.eq.0) then
          if (extrapbot.eq.0) then
            yval = ygrid(1)
          else
            extrapslope = (ygrid(2)-ygrid(1))/(xgrid(2)-xgrid(1))
            yval = ygrid(1) + extrapslope*(xval-xgrid(1))
          end if

        else ! a is hu-eg so we extrapolate
          if (extraptop.eq.0) then
            yval = ygrid(length)
          else
            extrapslope = (ygrid(length)-ygrid(length-1))/(xgrid(length)-xgrid(length-1))
            yval = ygrid(length) + extrapslope*(xval-xgrid(length))
          end if
        end if


      end subroutine linearinterpfromlocations

! ---------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!

! ---------------------------------------------------------------------------------------!
! Fully generic - need to write explanation
      subroutine gengrid(lowerbounds, upperbounds, numyears, gridpoints, growthgrid, assgrid)
        implicit none

! Arguments
        integer, intent (in) :: numyears
        integer, intent (in) :: gridpoints
        real (kind=rk), intent (in) :: lowerbounds(numyears)
        real (kind=rk), intent (in) :: upperbounds(numyears)
        real (kind=rk), intent (in) :: growthgrid

        real (kind=rk), intent (out) :: assgrid(gridpoints, (numyears+1))

! For program
        integer ixassnode, ixtoday
        real (kind=rk) restofgrid(gridpoints-8)

! Getting the asset grid
        do ixtoday = numyears, 1, -1
! I start the grid from the constraint plus the mininmum shock
          assgrid(1, ixtoday) = lowerbounds(ixtoday)
          assgrid(2, ixtoday) = assgrid(1, ixtoday) + 0.000001_rk
          assgrid(3, ixtoday) = assgrid(2, ixtoday) + 0.000003_rk
          assgrid(4, ixtoday) = assgrid(3, ixtoday) + 0.000005_rk
          assgrid(5, ixtoday) = assgrid(4, ixtoday) + 0.00001_rk
          assgrid(6, ixtoday) = assgrid(5, ixtoday) + 0.0001_rk
          assgrid(7, ixtoday) = assgrid(6, ixtoday) + 0.001_rk
          assgrid(8, ixtoday) = assgrid(7, ixtoday) + 0.005_rk

          call getnodes(restofgrid, assgrid(8,ixtoday)+0.01_rk, (upperbounds(ixtoday)), gridpoints-8, growthgrid)

          do ixassnode = 9, gridpoints 
            assgrid(ixassnode, ixtoday) = restofgrid(ixassnode-8)
          end do

        end do ! time loop

      end subroutine gengrid
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------!
! Fully generic 

      subroutine gengrid_extra(lowerbounds, breaks1, breaks2, upperbounds, numyears, gridpoints1, gridpoints2, &
          gridpoints3, growthgrid1, growthgrid2, growthgrid3, gridpoints, assgrid)
        implicit none

! Arguments
        integer, intent (in) :: numyears
        integer, intent (in) :: gridpoints1
        integer, intent (in) :: gridpoints2
        integer, intent (in) :: gridpoints3
        integer, intent (in) :: gridpoints
        real (kind=rk), intent (in) :: lowerbounds(numyears)
        real (kind=rk), intent (in) :: breaks1(numyears)
        real (kind=rk), intent (in) :: breaks2(numyears)
        real (kind=rk), intent (in) :: upperbounds(numyears)
        real (kind=rk), intent (in) :: growthgrid1
        real (kind=rk), intent (in) :: growthgrid2
        real (kind=rk), intent (in) :: growthgrid3

        real (kind=rk), intent (out) :: assgrid(gridpoints, numyears)

! For program
        integer ixassnode, ixtoday, index
        real (kind=rk) restofgrid1(gridpoints1-2)
        real (kind=rk) restofgrid2(gridpoints2+1)
        real (kind=rk) restofgrid3(gridpoints3+1)

! Check gridpoints
        if ((gridpoints1+gridpoints2+gridpoints3).ne.gridpoints) then
            write(*,*) gridpoints1, gridpoints2, gridpoints3, gridpoints
          write (*, *) 'Error in gengrid_extra; gridpoints1 + gridpoints2 + gridpoints3 is not equal gridpoints'
          stop
        end if

! Getting the asset grid
        do ixtoday = 1, numyears, 1

! I start the grid from the constraint plus the mininmum shock
          assgrid(1, ixtoday) = lowerbounds(ixtoday)
          assgrid(2, ixtoday) = assgrid(1, ixtoday) + 0.000001_rk

          call getnodes(restofgrid1, assgrid(2,ixtoday)+0.01_rk, breaks1(ixtoday), gridpoints1-2, growthgrid1)
          do ixassnode = 3, gridpoints1 - 1, 1 ! Don't want to add in break1 - leave that till next one
            assgrid(ixassnode, ixtoday) = restofgrid1(ixassnode-2)
          end do

          call getnodes(restofgrid2, breaks1(ixtoday), breaks2(ixtoday), gridpoints2+1, growthgrid2)
          index = 1
          do ixassnode = (gridpoints1), (gridpoints1+gridpoints2-1), 1
            assgrid(ixassnode, ixtoday) = restofgrid2(index)
            index = index + 1
          end do

          call getnodes(restofgrid3, breaks2(ixtoday), upperbounds(ixtoday), gridpoints3+1, growthgrid3)
          index = 1
          do ixassnode = (gridpoints1+gridpoints2), gridpoints, 1
            assgrid(ixassnode, ixtoday) = restofgrid3(index)
            index = index + 1
          end do

        end do ! time loop

      end subroutine gengrid_extra
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!

! This function returns a grid where:
! 1. The grid starts at firstpoint (input)
! 2. The grid ends at lastpoint (input)
! 3. The number of nodes is numberofnodes (input)
! 4. The growth rate of the step size is growthrate (input)

! This function is fully generic

      subroutine getnodes(nodes, firstpoint, lastpoint, numberofnodes, growthrate)
        implicit none

! Arguments
        real (kind=rk), intent (in) :: firstpoint
        real (kind=rk), intent (in) :: lastpoint
        real (kind=rk), intent (in) :: growthrate
        integer, intent (in) :: numberofnodes
        real (kind=rk), intent (out) :: nodes(numberofnodes)

! For programme
        real (kind=rk) step
        real (kind=rk) denom
        integer ix

! Check inputs
        if (firstpoint.ge.lastpoint) then
          write (*, *) firstpoint, lastpoint
          write (*, *) 'In getnodes, firstpoint must be strictly less than lastpoint'
          stop
        end if

! First I need, conditional, on the span of the space, the number of gridpoints, and the growth
! rate of the space between successive gridpoints, to work out the size of the first step
        denom = 0
        do ix = 1, numberofnodes - 1, 1
          denom = denom + (1+growthrate)**(ix-1)
        end do

        step = (lastpoint-firstpoint)/denom

        nodes(1) = firstpoint
        do ix = 2, numberofnodes, 1
          nodes(ix) = nodes(ix-1) + step
          step = step*(1+growthrate)
        end do

      end subroutine getnodes
! ---------------------------------------------------------------------------------------------------------!
! Function to get numerical first derivative. Generic other than use of linearinterp1 and I've hardcoded
! last two arguments in linearinterp1
      real (kind=rk) function deriv(x, fx, x0, length)
        implicit none

! Arguments
        integer, intent (in) :: length
        real (kind=rk), intent (in) :: x(length)
        real (kind=rk), intent (in) :: fx(length)
        real (kind=rk), intent (in) :: x0

! For progamme
        real (kind=rk) :: forderiv1, forderiv2

        call linearinterp1(x, fx, length, x0+0.0000001_rk, forderiv1, 0, 1)
        call linearinterp1(x, fx, length, x0-0.0000001_rk, forderiv2, 0, 1)
        deriv = (forderiv1-forderiv2)/(0.0000002_rk)

      end function deriv
! ---------------------------------------------------------------------------------------------------------!

! ---------------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------------!
! A subroutine for finding  the nodes and the values on a real grid that bound a value off the grid
! It is fully generic
      subroutine findingridr(xgrid, length, xval, nearestloc, lowerloc)
        implicit none
! Arguments
        integer, intent (in) :: length
        real (kind=rk), intent (in) :: xgrid(length)
        real (kind=rk), intent (in) :: xval

        integer, intent (out) :: nearestloc
        integer, intent (out) :: lowerloc

! For programme
        integer :: jl
        integer :: jm
        integer :: ju
        integer :: upperloc
        real (kind=rk) :: lowerval
        real (kind=rk) :: upperval
        intrinsic size

! Checking inputs
        if ((size(xgrid).ne.length)) then
          write (*, *) 'An argument in findingridr is not of the specified length'
          stop
        end if

! This block of code finds the two points on the xgrid between which the x value lies
        jl = 0
        ju = length + 1
        do while (ju-jl.gt.1)
          jm = (ju+jl)/2
          if (xval.gt.xgrid(jm)) then
            jl = jm
          else
            ju = jm
          end if
        end do

        lowerloc = jl
        upperloc = jl + 1

        if (lowerloc.eq.0) then
          lowerval = xgrid(1)
          upperval = xgrid(1)
        else if (lowerloc.eq.length) then
          lowerval = xgrid(length)
          upperval = xgrid(length)
        else
          lowerval = xgrid(lowerloc)
          upperval = xgrid(upperloc)
        end if

        if ((((upperval-xval).lt.(xval-lowerval)) .or. (lowerloc.eq.0)) .and. (lowerloc.ne.length)) then
          nearestloc = upperloc
        else
          nearestloc = lowerloc
        end if

        if ((nearestloc.lt.0) .or. (nearestloc.gt.length)) then
          write (*, *) 'error in findingr - nearestloc is outside the grid'
          stop
        end if

      end subroutine findingridr


! ---------------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------------!
! This subroutine takes a vector of shock values that represent a stationary Markov process
! and generates profiles of shock values using the associated cdf
! Fully generic
      subroutine simmarkovshocks(zvect, cdf, length, numperiods, nosims, startatmiddle, shockslist, shocksval)
        implicit none

! Arguments
        integer, intent (in) :: length ! These are the number the number of discrete points in the shock vector
        real (kind=rk), intent (in) :: zvect(length)
        real (kind=rk), intent (in) :: cdf(length, length)
        integer, intent (in) :: numperiods
        integer, intent (in) :: nosims
        logical, intent (in) :: startatmiddle

        real (kind=rk), intent (out) :: shocksval(numperiods, nosims)
        integer, intent (out) :: shockslist(numperiods, nosims)

! For use by subroutine
        integer, parameter :: ntforstat = 100 ! No of periods in simulation to findi stationary distribution
        real (kind=rk) :: uniformvar(nosims*numperiods)
        real (kind=rk) :: uniformvarforstat(nosims*ntforstat)
        integer :: zeroshockloc
        integer :: todayshockloc
        integer :: yesterdayshockloc
        integer :: statshockslist(nosims) ! Using 100 periods to find the stationary distribution (i.e. to get the
! initial shock for each person)
        integer igen, ifail, seed(4)
        integer :: lowerboundcdf
! Indexes
        integer ixtime, ixss, index
        integer :: state
! Scratch
        integer scratch1
        
! external
        !external g05kbf
        !external g05lgf
        external G05KFF

! Getting random (uniformly distributed) numbers
! This is for the burn in
        seed(1) = 559987891
        seed(2) = 357865655
        seed(3) = 357876222
        seed(4) = 981656871
        
        ! For older versions of NAG use the below
        !igen = 0
        !call g05kbf(igen, seed)
        !igen = 0
        !ifail = 0
        !call g05lgf(0.0_rk, 1.0_rk, nosims*ntforstat, uniformvarforstat, igen, seed, ifail)

ifail = 0
call G05KFF(1,1,seed(1),1,state,1,ifail)        
call g05sqf(nosims*ntforstat,0.0_rk,1.0_rk,state,uniformvarforstat,ifail)
        
        
        
        zeroshockloc = length/2 + 1 ! This will give the zero shock as long as length is odd
         !Check
        if ((zeroshockloc.lt.1).or.(zeroshockloc.gt.length)) then
          write (*,*) 'error in simmarkovshocks'
          write (*,*) 'zeroshockloc shock is not in range'
          write (*,*) 'Is length of shock vector 1'
          stop
        end if
 
 
        index = 0 ! Will run through the random number vector

! First finding the stationary distribution of shocks
        do ixss = 1, nosims
          do ixtime = 1, ntforstat
            if (ixtime.eq.1) then
              yesterdayshockloc = zeroshockloc
            else
              yesterdayshockloc = todayshockloc
            end if
            index = index + 1

            call findingridr(cdf(:,yesterdayshockloc), length, uniformvarforstat(index), scratch1, lowerboundcdf)
            todayshockloc = lowerboundcdf + 1
          end do ! ixtime
          statshockslist(ixss) = todayshockloc
        end do ! ixss

! Burn in over, lets now find the shocks for our simulatiion
        seed(1) = 465465465
        seed(2) = 894632231
        seed(3) = 123654982
        seed(4) = 888888888
        igen = 0
        call g05kbf(igen, seed)
        igen = 0
        ifail = 0
        call g05lgf(0.0_rk, 1.0_rk, nosims*numperiods, uniformvar, igen, seed, ifail)

        index = 0
        do ixss = 1, nosims
          do ixtime = 1, numperiods, 1
            if (ixtime.eq.1) then
              if (startatmiddle) then ! We want to start everyone from the zero shock
                yesterdayshockloc = zeroshockloc
              else
                yesterdayshockloc = statshockslist(ixss) ! Take the last shock for that individual in the last of
! period of the
              end if
            else ! of the synthethic period used above to find the stationary distribution
              yesterdayshockloc = todayshockloc
            end if
            index = index + 1

            call findingridr(cdf(:,yesterdayshockloc), length, uniformvar(index), scratch1, lowerboundcdf)
            todayshockloc = lowerboundcdf + 1
            shockslist(ixtime, ixss) = todayshockloc
            shocksval(ixtime, ixss) = zvect(todayshockloc) ! To look at the actual shocks
          end do ! ixtime
        end do ! ixss
      end subroutine simmarkovshocks
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!

! This subroutine takes a vector of shock values that represent a stationary Markov process
! and generates profiles of shock values using the associated cdf
! Fully generic
      subroutine simmarkovshocks_nonstat(zvect, cdf, length, numperiods, startingshock,nosims, shockslist, shocksval)
        implicit none

! Arguments
        integer, intent (in) :: length ! These are the number the number of discrete points in the shock vector
        integer, intent (in) :: numperiods
        integer, intent (in) :: startingshock
        real (kind=rk), intent (in) :: zvect(length, numperiods)
        real (kind=rk), intent (in) :: cdf(length, length, numperiods)
        integer, intent (in) :: nosims

        real (kind=rk), intent (out) :: shocksval(numperiods, nosims)
        integer, intent (out) :: shockslist(numperiods, nosims)

! For use by subroutine
        real (kind=rk) :: uniformvar(nosims*numperiods)
        integer :: zeroshockloc
        integer :: todayshockloc
        integer :: yesterdayshockloc
        integer igen, ifail, seed(4)
        integer :: lowerboundcdf
! Indexes
        integer ixtime, ixss, index

! Scratch
        integer scratch1

! external
        external g05kbf
        external g05lgf
        
        !Check
        if ((startingshock.lt.1).or.(startingshock.gt.length)) then
          write (*,*) 'error in simmarkovshocks_nonstat'
          write (*,*) 'starting shock is not in range'
          stop
        end if

        zeroshockloc =  startingshock
        index = 0 ! Will run through the random number vector

! Lets now find the shocks for our simulatiion
        seed(1) = 465465465
        seed(2) = 894632231
        seed(3) = 123654982
        seed(4) = 888888888 !cf. Nick Leeson
        igen = 0
        call g05kbf(igen, seed)
        igen = 0
        ifail = 0
        call g05lgf(0.0_rk, 1.0_rk, nosims*numperiods, uniformvar, igen, seed, ifail)

        index = 0
        do ixss = 1, nosims
          do ixtime = 1, numperiods, 1
            if (ixtime.eq.1) then
              yesterdayshockloc = zeroshockloc
            else
              yesterdayshockloc = todayshockloc
            end if
            index = index + 1

            call findingridr(cdf(yesterdayshockloc,:,ixtime), length, uniformvar(index), scratch1, lowerboundcdf)
            todayshockloc = lowerboundcdf + 1
            shockslist(ixtime, ixss) = todayshockloc
            shocksval(ixtime, ixss) = zvect(todayshockloc, ixtime) ! To look at the actual shocks
          end do ! ixtime
        end do ! ixss

      end subroutine simmarkovshocks_nonstat
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!


! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
! Check stochastic discount factor with with one-dimensional vector
      subroutine checksdf(sdf, licon, dim1, numtests, testvalues, indincnonnull, indincnoncon, countfail, numfails, &
          failprop)
        implicit none

        integer, intent (in) :: dim1
        integer, intent (in) :: numtests
        real (kind=rk), intent (in) :: sdf(dim1)
        integer, intent (in) :: licon(dim1)
        real (kind=rk), intent (in) :: testvalues(numtests)
        integer, intent (out) :: indincnonnull(dim1)
        integer, intent (out) :: indincnoncon(dim1)
        integer, intent (out) :: countfail(dim1, numtests)
        integer, intent (out) :: numfails(numtests)
        real (kind=rk), intent (out) :: failprop(numtests)

        integer :: ixtest, numnoncons

        where (licon.ne.2)
          indincnonnull = 1
        elsewhere
          indincnonnull = 0
        end where

        where ((licon.ne.2) .and. (licon.ne.1))
          indincnoncon = 1
        elsewhere
          indincnonnull = 0
        end where

        do ixtest = 1, numtests, 1
          where ((indincnoncon.eq.1) .and. ((sdf.lt.(1.0_rk-testvalues(ixtest))) .or. (sdf.gt. &
              (1.0_rk+testvalues(ixtest)))))
            countfail(:, ixtest) = 1
          elsewhere
            countfail(:, ixtest) = 0
          end where
        end do

!Getting sums
        numfails = sum(countfail, 1)
        numnoncons = sum(indincnoncon, 1)

        failprop = numfails/numnoncons

      end subroutine checksdf
! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!





! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
! A routine that takes a vector representing a PDF and returns a vector representing a CDF
      subroutine pdftocdf(length, pdf, cdf)
        implicit none
        integer, intent (in) :: length
        real (kind=rk), intent (in) :: pdf(length)
        real (kind=rk), intent (out) :: cdf(length)

! For programme
        integer :: ixlen
        real (kind=rk) :: sumforcdf

        if ((sum(pdf).lt.0.999) .or. (sum(pdf).gt.1.001)) then
          write (*, *) 'error in pdftocdf: probabilities in pdf do not sum to 1'
          write (*, *) 'sum is', sum(pdf)
          stop
        end if

! Get CDF
        sumforcdf = 0
        do ixlen = 1, length
          sumforcdf = sumforcdf + pdf(ixlen)
          cdf(ixlen) = sumforcdf
        end do

      end subroutine pdftocdf
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
!Get uniform draws using the NAG libraries
      subroutine getdraws(length1, length2, totalrandom, seed, draws, normal, mu, var)
        implicit none

        integer, intent (in) :: length1, length2
        integer, intent (in) :: seed
        integer, intent (in) :: totalrandom
        logical, intent (in), optional :: normal
        real (kind=rk), intent (in), optional :: mu
        real (kind=rk), intent (in), optional :: var
        real (kind=rk), intent (out) :: draws(length1, length2)

!For programme
        integer :: genid
        integer :: subid !Says use the NAG basic generator
        integer :: lseed
        integer, parameter :: lstate = 633
        real (kind=rk) :: drawsvec(totalrandom)
        integer :: state(lstate)
        integer :: ifail
        external :: g05kff, g05saf

!Check that if we have requested normal variables, mu and var are provided
        if (present(normal)) then
          if ((normal) .and. (.not. (present(mu) .and. present(var)))) then
            write (*, *) 'error in getdraws. If normal is .true.,  mean and variance must be supplied'
            stop
          end if
        end if

!Check length1 * length2 = totalrandom
        if ((length1*length2).ne.totalrandom) then
          write (*, *) 'error in getunifdraws: length1 * length2 = totalrandom'
          stop
        end if

        genid = 1 !Says use the NAG basic generator; do not change - will not work otherwise; need to write new subroutine if want to use more genera
 !
!
!
!l
        subid = 1 !This is irrelavant when we're using the NAG basic generator (i.e. genid = 1)
        lseed = 1 !size of seed	
        ifail = 0

! Initialize the generator to a repeatable sequence
        call g05kff(genid, subid, seed, lseed, state, lstate, ifail)

        ifail = 0

        if (present(normal) .and. (normal)) then
          call g05skf(totalrandom, mu, var, state, drawsvec, ifail)
        else
          call g05saf(totalrandom, state, drawsvec, ifail)
        end if

        draws = reshape(drawsvec, (/length1,length2/) )


      end subroutine getdraws

! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!

! ---------------------------------------------------------------------------------------------------------!
! ---------------------------------------------------------------------------------------------------------!
!Function that gets the squared distance between two points
      function dist2(x, y)
        implicit none
        real (kind=rk), intent (in) :: x, y
        real (kind=rk) :: dist2

        dist2 = (x-y)*(x-y)

      end function dist2


!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
!A subroutine for trilinear interpolation
!A nice method here (that I'm not using) method at: http://paulbourke.net/miscellaneous/interpolation/
!This has been tested with the subroutine test below
      subroutine linearinterp3(x1grid, x2grid, x3grid, dim1, dim2, dim3, x1val, x2val, x3val, yval, ygrid)
        use types
        implicit none

! Arguments
        integer, intent (in) :: dim1
        integer, intent (in) :: dim2
        integer, intent (in) :: dim3
        real (kind=rk), intent (in) :: x1grid(dim1)
        real (kind=rk), intent (in) :: x2grid(dim2)
        real (kind=rk), intent (in) :: x3grid(dim3)
        real (kind=rk), intent (in) :: ygrid(dim1, dim2, dim3)
        real (kind=rk), intent (in) :: x1val
        real (kind=rk), intent (in) :: x2val
        real (kind=rk), intent (in) :: x3val
        real (kind=rk), intent (out) :: yval

! For programme
        integer :: x3lowerloc
        integer :: scratch_nearestloc
        integer :: newxgridlowerloc
        real (kind=rk) :: newygrid(2)
        real (kind=rk) :: newxgrid(2)

        if ((size(x1grid).ne.dim1) .or. (size(x2grid).ne.dim2) .or. (size(x3grid).ne.dim3)) then
          write (*, *) 'An argument in linearinterp3 is not of the specified length'
          stop
        end if

! Locate x3val on the grid
          call findingridr(x3grid, dim3, x3val, scratch_nearestloc, x3lowerloc)

! Find if any of the values are off the grid       
          if (x3lowerloc.eq.0) then
            x3lowerloc = 1
          end if

          if (x3lowerloc.eq.dim3) then
            x3lowerloc = dim3 - 1
          end if


!Do two two bilinear interpolations over x1grid and x2grid keeping  x3 at x3lowerloc and x3lowerloc + 1 
          call linearinterp2(x1grid, x2grid, dim1, dim2, x1val, x2val, newygrid(1), ygrid(:,:,x3lowerloc))
          call linearinterp2(x1grid, x2grid, dim1, dim2, x1val, x2val, newygrid(2), ygrid(:,:,(x3lowerloc+1)))

! Now to a linear interpolation on these found vlues
! First generate a new grid from from the x2 bounds (a new ygrid has already bee generated)
        newxgrid(1) = x3grid(x3lowerloc)
        newxgrid(2) = x3grid(x3lowerloc+1)

!Think this is redundant
        if (x3lowerloc.eq.0) then
          newxgridlowerloc = 0
        else if (x3lowerloc.eq.dim3) then
          newxgridlowerloc = 2
        else
          newxgridlowerloc = 1
        end if

        call linearinterpfromlocations(2,newxgrid, newygrid, newxgridlowerloc, x3val,  1, 1, yval)


      end subroutine linearinterp3

! ---------------------------------------------------------------------------------------------------------!

subroutine getannuityratesnew(maxyears,survmale,survfemale,intrate,annuityrate, penpropinherit, annAdminLoad)
  
  implicit none

  integer, intent(in) :: maxyears
  real (kind=rk), intent(in) :: survmale(maxyears)
  real (kind=rk), intent(in) :: survfemale(maxyears)
  real (kind=rk), intent(in) :: intrate
  real (kind=rk), intent(out) :: annuityrate(maxyears, 3)
  real (kind=rk), intent(in) :: penpropinherit
  real (kind=rk), intent(in) :: annAdminLoad
  
  real (kind=rk) :: capitalf(maxyears, 3) !This and the following follow the notation in my notes
  
  
  !For program
  integer :: ixtoday
  integer :: tau
  
  
  capitalf(maxyears, 1) = 0.0_rk
  capitalf(maxyears, 2) = 0.0_rk
  capitalf(maxyears, 3) = 0.0_rk
  
  
  !Elsewhere in the programme - I interpret survprob(ixtoday) as the probability as surviving
  !until ixtoday + 1. So, notwithstanding the fact that in my notes I have written this object
  !as s_{ixtoday + 1) I follow the convention here. 
  
  do ixtoday = maxyears-1, 1, -1
     tau = maxyears - ixtoday !years left
  
     !Couple
      capitalf(ixtoday, 1)= (survmale(ixtoday)*survfemale(ixtoday)*(1.0_rk +  capitalf(ixtoday+1, 1)) &
        + penpropinherit * survmale(ixtoday) * (1.0_rk - survfemale(ixtoday)) * (1.0_rk + capitalf(ixtoday+1, 2)) &
        + penpropinherit * survfemale(ixtoday) * (1.0_rk - survmale(ixtoday)) * (1.0_rk + capitalf(ixtoday+1, 3)) )/(1+intrate) 
  
      annuityrate(ixtoday, 1) = 1.0_rk/( ((1.0_rk)* (1.0_rk + annAdminLoad)) * capitalf(ixtoday, 1))
      
    
  !Male
       capitalf(ixtoday, 2)  =   (survmale(ixtoday) * (1.0_rk + capitalf(ixtoday+1, 2)) ) / (1 + intrate)        
       annuityrate(ixtoday, 2) = 1.0_rk/( ((1.0_rk)* (1.0_rk + annAdminLoad)) *  capitalf(ixtoday, 2))
     
  !Female
      capitalf(ixtoday, 3) =   (survfemale(ixtoday) * (1.0_rk + capitalf(ixtoday+1, 3)) )/(1+intrate)        
      annuityrate(ixtoday, 3) = 1.0_rk/(((1.0_rk)* (1.0_rk + annAdminLoad)) * capitalf(ixtoday, 3))
        
  end do

annuityrate(maxyears, :) = (huge(annuityrate(maxyears, 1)))/10.0_rk
  
    
end subroutine getannuityratesnew
! ---------------------------------------------------------------------------------------------------------!

    end module routines_generic
