    module types_generic
      use precisions
      use globalvalues
      
!---------------------------------------------------------------------------------------------------------!
      type earnproc
        real (kind=rk) :: params(3)
        real (kind=rk) :: ar1rho
        real (kind=rk) :: ar1mean
        real (kind=rk) :: ar1stdev
      end type earnproc
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type taxtype

        ! Income tax  
        real (kind=rk) :: tfa
        real (kind=rk) :: tfa6574
        real (kind=rk) :: tfa75plus
        real (kind=rk) :: startinglimit
        real (kind=rk) :: higherthreshold
        real (kind=rk) :: startingrate
        real (kind=rk) :: basicrate
        real (kind=rk) :: higherrate
        
        ! NI
        real (kind=rk) :: lel
        real (kind=rk) :: uel
        real (kind=rk) :: NIlower
        real (kind=rk) :: NIhigher
        
        ! Child benefit
        real (kind=rk) :: childBenRate1
        real (kind=rk) :: childBenRate2
        
        ! State pension age
        integer :: statepenage

        ! Tax interest
        logical :: taxInterest
        
      end type taxtype
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type quad
        integer :: dim
        real (kind=rk), allocatable :: weigh(:)
        real (kind=rk), allocatable :: abs(:)
      end type quad
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type utilityparams
       sequence
        real (kind=rk) :: beta
        real (kind=rk) :: gamma
        real (kind=rk) :: oneover1minusgamma
        real (kind=rk) :: oneminusgamma
        

         
        real (kind=rk) :: equivscaleinvvec(3) !One for each marital status                
        real (kind=rk) :: equivscaleinv  !Used for inplay equivscaleinv
        real (kind=rk) :: equivscale !Used for equivscael        
        
        logical :: multFelByEquiv
        

                
      end type utilityparams
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type pcsys
        !Relevant to both
        real (kind=rk) :: level 
        real (kind=rk) :: capflowrate
        real (kind=rk) :: capdisreg      
        
        !Relevant to Guarantee credit
        logical :: guarcredon
        integer :: eligage
         
        !Relevant to savings credit
        integer :: sceligage
        logical :: savcredon 
        real (kind=rk) :: maxincforsavcred  
        real (kind=rk) :: maxsavcred
        real (kind=rk) :: taper 
        real (kind=rk) :: oneminustaper  
        real (kind=rk) :: scthreshold   
        
        
        !For couple
        real (kind=rk) :: levelcoup
        real (kind=rk) :: maxincforsavcredcoup
        real (kind=rk) :: maxsavcredcoup
        real (kind=rk) :: scthresholdcoup   

      end type pcsys

!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!

      type grids

!---!
!Information for grids
!---!
        integer :: ass
        integer :: dc
        integer :: pen
        integer :: ar1
        integer :: ar1by2
        integer :: ar1by3
        integer :: ar1by4
        integer :: cons
        integer :: numhhcomp
        integer :: lowerar1nodeforint
        integer :: upperar1nodeforint
        integer :: housPoints        
        
!---!
!Information for optimisation and for programme
!---!
        real (kind=rk) :: goldentol
        integer :: sims
        integer :: optimMethod

!---!
!Retirement and pension
!---!
        integer :: statepenage
 !      integer :: numdcch
        real (kind=rk) :: maxdccont
        real (kind=rk) :: dcmeanret
        real (kind=rk) :: dcstdev
        real (kind=rk) :: dclumpsumprop
        real (kind=rk) :: oneminusdclumpsumprop
        type (quad) :: dcquad
        real (kind=rk), allocatable :: fairannuity(:, :)
        real (kind=rk) :: penpropinherit
        real (kind=rk) :: annAdminLoad
    integer :: numdcch    
!---!
!Environment
!---!
        integer :: numyears
        type (taxtype) :: tax2005
        real (kind=rk), dimension(numyears) :: survprobs_m, survprobs_f
        real (kind=rk) :: survprobs3way(3, 4,numyears)
        real (kind=rk) :: uncondtrans
        real (kind=rk) :: rate
        real (kind=rk), dimension(numyears) :: numeqkids, numkids
        real (kind=rk), allocatable :: manshareearnings(:)
        logical :: useequivscale      
        logical :: usesurvprob
        integer :: firstageatwhichdeath
    
!---!
!Earnings
!---!
        type (earnproc) :: earn
        type (earnproc) :: earnPop
        real (kind=rk) :: womanearnprop
        logical (kind=rk) :: nosepwomanearn              
        
        
!---!
!DC choices
!---!	
        real (kind=rk), allocatable :: alldcchoice(:)

!---!
!Utility function parameters
!---!	
      type (utilityparams) :: params

!---!
!Pension credit system
!---!	 
      type (pcsys) :: pc


!---!
!Unemployment
!---!	 
      real (kind=rk) :: unemprisk
      real (kind=rk) :: unemppay
      real (kind=rk) :: womannoworkprob(2)
      
!---!
!Are we estimating or iterating
!---!	 
     logical :: nowiterating

!---!
!Labour supply 
!---!	 
    logical :: dontchecksdfs !Should we check things
    logical :: dontdovalfuncsigncheck !Should we check things
    logical :: integrateoutsideoptimisation !Whether we integrate in the central bit of the optimisation routine   
    logical :: worktorule !Whether to do all the checks etc
    logical :: equivHousingFlow ! Equivilise housing flow?
    real (kind=rk) :: multOnHousingFlow
    
!----!
!Things for optimality project
!----!
    real (kind=rk), allocatable :: manshare(:)    


    real (kind=rk) :: sparefordev ! A spare variable for use in development
    real (kind=rk) :: unempKids

    real (kind=rk), allocatable  :: popEarQuantsAt64(:)
    real (kind=rk) :: FEdecile
    integer :: edType
    real (kind=rk) :: dbcoeffs(6), statepencoeffs(6)


!------!
!Rates of  return
!------!

real (kind=rk) :: mortR, housCG, housYield !housR,
real (kind=rk) :: portb0, portbage, portbage2, portwealth, portwealth2, porthaveclaimed
real (kind=rk) :: lev0, lev1, lev2
integer :: intMeth
logical :: hasDB, hasHous

 
      end type grids

      
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type policy_funcs
        real (kind=lowrk), allocatable :: c_pre(:, :, :, :, :) 
        real (kind=lowrk), allocatable :: c_post(:,  :, :, :, :, :)                  
        integer (kind=midprecintkind), allocatable :: dc_pre(:, :, :, :, :) 
      end type policy_funcs
!---------------------------------------------------------------------------------------------------------!
      type val_funcs
        real (kind=rk), allocatable ::  pre(:,  :,  :, :, :)
        real (kind=rk), allocatable :: post(:,  :,  :, :, :, :)  
        real (kind=rk), allocatable :: mu_pre(:,  :,  :, :, :)
        real (kind=rk), allocatable :: mu_post(:,  :,  :, :, :, :)           
        real (kind=rk), allocatable :: emu_pre(:,  :,  :, :, :)
        real (kind=rk), allocatable :: emu_post(:,  :,  :, :, :, :)           
      end type val_funcs
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
      type typetomgrids
        real (kind=rk), allocatable :: ass(:)
        real (kind=rk), allocatable :: pen(:)
        real (kind=rk), allocatable :: cum(:)
        real (kind=rk), allocatable :: dc(:)
        real (kind=rk), allocatable :: hous(:)
      end type typetomgrids
!---------------------------------------------------------------------------------------------------------!
      type typetomstates
        real (kind=rk):: cash
        real (kind=rk):: dc
        real (kind=rk):: pen
        real (kind=rk):: cum
        real (kind=rk):: hous
        real (kind=rk):: windfall
      end type typetomstates
!---------------------------------------------------------------------------------------------------------!   
      type typetomvalfuncs
        real (kind=rk), allocatable :: val_ar1Int_pre_adc(:,:,:)        
        real (kind=rk), allocatable :: eval_pre_adlq(:,:,:,:)        
        real (kind=rk), allocatable :: val_post_apc(:,:,:)
        real (kind=rk), allocatable :: val_post_adch(:,:,:,:)
      
        real (kind=rk), allocatable :: mu_arInt_pre_adc(:,:,:)        
        real (kind=rk), allocatable :: linEmu_pre_adlq(:,:,:,:)        
        real (kind=rk), allocatable :: mu_post_apc(:,:,:)
        real (kind=rk), allocatable :: linEmu_post_ap(:,:)
        real (kind=rk), allocatable :: mu_post_apch(:,:,:,:)
        real (kind=rk), allocatable :: linEmu_post_aph(:,:,:)

      end type typetomvalfuncs
 !---------------------------------------------------------------------------------------------------------!   
      type typetodayinfo
         integer :: leistoday
         integer :: ixleis
         integer :: ixdcchoice
         logical :: dcalreadyintegrated
         integer :: ixhhcomp
         integer :: ixtoday
         integer :: ixage
         integer :: ixassnode 
         integer :: ixdcnode
         integer :: ixar1node   
         integer :: ixpenpoint  
         integer :: ixhouspoint     
         integer :: ixclaimchoice    
         real (kind=rk) :: numkids
         
         ! Housing stuff
         ! Today state pensions 
         real (kind=rk) :: statepen
         
         ! DB
        real (kind=rk) :: db

        real (kind=rk) :: shockDecile
         
        real (kind=rk) :: leverage       
        real (kind=rk) :: over65
        real (kind=rk) :: housingIfOver65

        real (kind=rk) :: margR        
        
        integer :: yearsFrom2002
      end type typetodayinfo
   
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!
 type typeopt_realisations

    real (kind=rk) :: realisedearnings(numyears)
    real (kind=rk) :: d_realisedearnings(numyears)
    real (kind=rk) :: manshare(numyears)
    integer :: d_earningnode(numyears)    
    integer :: realisedunemp(numyears)
    real (kind=rk) :: unemptv(numyears)
    real (kind=rk) :: dcgr(numyears) !DC return
    integer :: yob
    real (kind=rk) :: penData, cashData, housData, npData
    

end type typeopt_realisations        
!---------------------------------------------------------------------------------------------------------!
!---------------------------------------------------------------------------------------------------------!

    end module types_generic
