/******************************************************************************/
* Crawford and O'Dea
* "Household portfolios and financial preparedness for retirement" 
*  
* Replicated .do file for production of results in section 5 
*
* This do file pulls in simulated model output and combines  into long dataset 
* Run this file first, then run the analysis file
*  						                 
/******************************************************************************/


pause on
clear
clear matrix
clear mata
set more off
set maxvar 20000

*Where should the model pull the data in from?
local modir  "P:\adandopt\data\modeloutput\30August2018"

*Where should the data be saved?
local tempdir "P:\adandopt\data\modelanalysis\30August2018\data"

*What's the link between the model and elsa?
local elsalink "P:\adandopt\data\modelanalysis\Uta\31Jan2018\idauniq_link_995.dta"

local allspecs  1 2 3 4 5 6 7 8 9 11 12 13 14 15 16 17 18 20 21 22 23 24 25 26 27 28 29 30 31 32 33 37 40    //Model specifications
local minspec 1 
local otherspecs 2 3 4 5 6 7 8 9 11 12 13 14 15 16 17 18 20 21 22 23 24 25 26 27 28 29 30 31 32 33 37 40  

local persnum 995 

/******************************/
/* Bring in model output data */
/******************************/
*Note: Data sets conmean, incmean, housingPathsmean, dcstockvalmean,assStartmnean are 81 x persnum. 
*I import them as such and then transpose them to be persnum x 81 to then match the discount factor which is persnum x 3 
*Then, I change the data sets in long form
    
foreach spec of local allspecs {
	
	/****************************************/
    /*Read in consumption*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\conmean.out", clear 
	xpose,clear
	rename v* conmean`spec'_*
    gen tempid = _n
    save `tempdir'\conmean`spec', replace
	/****************************************/
    /*Read in gross earnings*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\earnmean.out", clear
	xpose,clear
	rename v* earnmean`spec'_*
    gen tempid = _n
    save `tempdir'\earnmean`spec', replace
	/****************************************/
    /*Read in net income*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\incmean.out", clear
	xpose,clear
	rename v* incmean`spec'_*
    gen tempid = _n
    save `tempdir'\incmean`spec', replace
	/****************************************/
    /*Read in housing value*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\housingPathsmean.out", clear
	xpose,clear
	rename v* housing`spec'_*
    gen tempid = _n
    save `tempdir'\housing`spec', replace
	/****************************************/
    /*Read in consumption flow of housing   */
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\housingYieldmean.out", clear
	xpose,clear
	rename v* housingyield`spec'_*
    gen tempid = _n
    save `tempdir'\housingyield`spec', replace
	/****************************************/
    /*Read in DC pension stock */
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\dcstockvalmean.out", clear
	xpose,clear
	rename v* dcpen`spec'_*
    gen tempid = _n
    save `tempdir'\dcpen`spec', replace
	/****************************************/
    /*Cash*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\assStartmean.out", clear
	xpose,clear
	rename v* cash`spec'_*
    gen tempid = _n
    save `tempdir'\cash`spec', replace
	/****************************************/
    /*State pension income*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\statepenmean.out", clear
	xpose,clear
	rename v* statepenmean`spec'_*
    gen tempid = _n
    save `tempdir'\statepenmean`spec', replace
	/****************************************/
    /*Defined benefit income*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\dbmean.out", clear
	xpose,clear
	rename v* dbmean`spec'_*
    gen tempid = _n
    save `tempdir'\dbmean`spec', replace
	/****************************************/
    /*Defined contribution income*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\dcmean.out", clear
	xpose,clear
	rename v* dcmean`spec'_*
    gen tempid = _n
    save `tempdir'\dcmean`spec', replace
	/****************************************/
    /*Gross income*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\grossincmean.out", clear
	xpose,clear
	rename v* grossincmean`spec'_*
    gen tempid = _n
    save `tempdir'\grossincmean`spec', replace
	/****************************************/
    /*Pension credit I*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\guarcredmean.out", clear
	xpose,clear
	rename v* guarcredmean`spec'_*
    gen tempid = _n
    save `tempdir'\guarcredmean`spec', replace
	/****************************************/
    /*Pension credit II*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\savcredmean.out", clear
	xpose,clear
	rename v* savcredmean`spec'_*
    gen tempid = _n
    save `tempdir'\savcredmean`spec', replace
	/****************************************/
    /*Cash Interest*/
    /****************************************/
    infile temp1-temp`persnum' using "`modir'\Spec `spec'\cashInterestmean.out", clear
	xpose,clear
	rename v* cashInterestmean`spec'_*
    gen tempid = _n
    save `tempdir'\cashInterestmean`spec', replace
	/****************************************/
    /*Discount factor*/
    /****************************************/
	infile tempid beta`spec' zeroone using "`modir'\Spec `spec'\foundBetas.txt", clear
    save `tempdir'\beta`spec', replace
	/***************************************************************/
	/* Year of birth - so that using exactly the same as the model */
	/***************************************************************/
	infile yob using "P:\adandopt\data\modelinput\modelinput_a\yob.txt", clear
	gen tempid=_n
	save `tempdir'\yob`spec', replace
	
    /****************************************/
    /*Merge together and make sure they match*/
    /****************************************/
	use `tempdir'\conmean`spec', replace
	merge 1:1 tempid using `tempdir'\housing`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\housingyield`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\incmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\earnmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\dcpen`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\cash`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\statepenmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\dbmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\dcmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\guarcredmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\savcredmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\cashInterestmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\grossincmean`spec', assert(3)
    drop _m
	merge 1:1 tempid using `tempdir'\beta`spec'
    drop _m
	merge 1:1 tempid using `tempdir'\yob`spec'
	drop _m
	
	/**************************/
	/* Reshape to long format */
	/**************************/
	reshape long conmean`spec'_ housing`spec'_ housingyield`spec'_ incmean`spec'_ earnmean`spec'_ dcpen`spec'_ cash`spec'_ statepenmean`spec'_ ///
	dbmean`spec'_ dcmean`spec'_ savcredmean`spec'_ guarcredmean`spec'_ cashInterestmean`spec'_  grossincmean`spec'_, i(tempid) j(year)
	
	
	/**************************************************************/
	/* SUBTRACT HOUSING WEALTH FOR THOSE YOUNGER THAN 65 IN MODEL */
	/**************************************************************/
	gen age_m =year+19
	gen   cash`spec'_old =cash`spec'_ /*keep copy to check*/
	replace cash`spec'_=cash`spec'_- housing`spec'_ if age_m<65
	
	save `tempdir'\data_for_analysis`spec'_long, replace
	}

/************************************************************/	
/* Get an appended version of the data with all specs in it */
/************************************************************/
foreach spec of local allspecs {
	use `tempdir'\data_for_analysis`spec'_long, replace
	gen spec = `spec'
	foreach var in conmean housing housingyield incmean dcpen cash statepenmean dbmean dcmean guarcredmean savcredmean cashInterestmean grossincmean earnmean {
		rename `var'`spec'_ `var'
		}
	rename beta`spec' beta
	rename cash`spec'_old cash_old
	save `tempdir'\data_for_analysis`spec'_longb, replace
	}
	
use `tempdir'\data_for_analysis`minspec'_longb, replace
foreach spec of local otherspecs {
	append using `tempdir'\data_for_analysis`spec'_longb
	}
save `tempdir'\data_for_analysis_all, replace

/****************************************/
* Create a measure of lifetime earnings */
/****************************************/

bys tempid spec: egen LTearn = sum(earn) if earn~=.
gen LTearnd=.
foreach spec in `allspecs' {
	xtile LTearnd_`spec' = LTearn if spec==`spec' , nq(10)
	replace LTearnd = LTearnd_`spec' if spec==`spec'
	drop LTearnd_`spec'
	}
bys tempid spec: egen LTearnvol1 = sd(earn) if earn~=.
bys tempid spec: egen LTearnvol2 = mad(earn) if earn~=.
bys tempid spec: egen LTearnvol3 = mdev(earn) if earn~=.
	
save `tempdir'\data_for_analysis_all, replace

	
/**********************/
/* MERGE IN ELSA DATA */
/**********************/

******************************************
*merge link for idauniq in elsa
merge m:1 tempid using `elsalink', assert(3)
drop _m
save `tempdir'\data_for_analysis_all, replace
/****************************************/

	*Wave 1
	local elsadir "M:\ELSA_core\data" 
	local otherdir "P:\adandopt\data"
	merge m:1 idauniq using  "`elsadir'\wave_1_core_data_v3.dta", keepusing(explan exlo80)
	keep if _m==3
	drop _m
	merge m:1 idauniq using "`elsadir'\wave_1_ifs_derived_variables.dta", keepusing(smokerstat smoker schleave chage*)
	drop if _merge!=3 /*Elsa contains more individuals than the ones we use*/
	drop _m
		gen numkids = 0
		forv x = 1/16 {
			replace numkids = `x' if chage`x'~=-1
			}
		drop chage*	
	merge m:1 idauniq using "`elsadir'\wave_1_financial_derived_variables.dta", keepusing(nettotnhw_bu_s nethw_bu_s )
	rename nettotnhw_bu_s nettotnhw_bu_s_w1 
	rename nethw_bu_s  nethw_bu_s_w1 
	drop if _merge!=3 /*Elsa contains more individuals than the ones we use*/
	drop _m
	*Wave 5 (last wave pension data is currently derived for)
	merge m:1 idauniq using "`elsadir'\wave_5_ifs_derived_variables.dta", keepusing(age)
	drop if _merge==2 /*Drop only the additional ones in elsa. Notice some people attrit. Keep those that attrit*/
	drop _m
	rename age age_obs_wave5
	merge m:1 idauniq using "`elsadir'\wave_5_financial_derived_variables.dta", keepusing(nettotnhw_bu_s nethw_bu_s )
	rename nettotnhw_bu_s nettotnhw_bu_s_w5
	rename nethw_bu_s  nethw_bu_s_w5 
	drop if _merge==2  /*Drop only the additional ones in elsa. Notice some people attrit. Keep those that attrit*/
	drop _m
	* Merge in ELSA w8 risk/time preference data
	merge m:1 idauniq using "`elsadir'\ELSAW8_Collaborators_Archive_2018_02_13.dta", keepusing(scgrisk scfrisk scgpati scfpati)
	keep if _m~=2
	drop _m
	* Merge in 9 types
	merge m:1 idauniq using "`otherdir'\type9.dta"
	keep if _m~=2
	drop _m
	/****************************************/
	*TO READ IN PENSION WEALTH (INCLUDES AGE VARIABLE - I checked, this is the same as in elsa wave 1)
	merge m:1 idauniq using "`otherdir'\penwealth_bu.dta" 
	drop if _merge!=3 /*Elsa contains more individuals than the ones we use*/
	drop _m
	rename priDCwealth_bu priDCwealth_bu_w1
	rename penwealth_bu penwealth_bu_w1
	rename priDBwealth_bu priDBwealth_bu_w1 
	rename age age_obs
	merge m:1 idauniq using "`otherdir'\DCpension_w5.dta"
	drop if _m==2
	drop _m
	rename priDCwealth_bu priDCwealth_bu_w5
	/****************************************/
	
save `tempdir'\data_for_analysis_all_elsa, replace
