/*********************************************************************/
* Crawford and O'Dea
* "Household portfolios and financial preparedness for retirement" 
*  
* Replicated .do file for production of results in section 5 
*
* This do file uses the dataset produced by getdata.do
* It produces the tables and figures of results in the paper
*  						                 
/*********************************************************************/  						                 

clear 
set more off
adopath +P:\ado

global resultsrun 30August2018

cd "P:\adandopt\data\modelanalysis\\$resultsrun\stataoutput"
global output "P:\adandopt\data\modelanalysis\\$resultsrun\stataoutput"
global data "P:\adandopt\data\modelanalysis\\$resultsrun\data"


/*--------------------------------*/
/* Bring in data in for all specs */
/*--------------------------------*/
* Note: This is created by getdata_2018_inclw5
use "$data\data_for_analysis_all_elsa.dta", replace

* Drop those with truncated discount rates
gen decentbeta1 = (beta>0.49 & beta<1.5)
gen decentbeta2 = (beta>0.8 & beta<1.2)
tabout spec using "$output\betatruncation.tex", sum c(mean decentbeta1 mean decentbeta2) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)

drop if (beta<=0.49 | beta==1.5) 


/*--------------------------------------------------------------*/
/* Analysis that just uses year of match between data and model */
/*--------------------------------------------------------------*/

gen modelagedata = 2002 - yob    /* This is the age the model thinks you are when you are observed in the data */
keep if age_m == modelagedata
save "$data\data_for_analysis_all_elsa_obsage.dta", replace
use "$data\data_for_analysis_all_elsa_obsage.dta", replace

	/*.........................................................*/
	/* Check that what is meant to be fitted is in fact fitted */
	/*.........................................................*/

	* For models 1,2,4, 5, 6, 8, 9, 28, 29 by construction we should have data(cash+pen+housing)=model(cash+pen+housing)
	gen 	dataformatch = (nettotnhw_bu_s_w1 + priDCwealth_bu_w1 + nethw_bu_s_w1)/1000  if                
	replace dataformatch = (nettotnhw_bu_s_w1 + penwealth_bu_w1 + nethw_bu_s_w1)/1000    if (spec==1 | spec==2 | spec==3 | spec==4 | spec==9 | spec==11 | spec==12 | spec==13 | spec==14 | (spec>=20 & spec<=29))


	gen simformatch = (cash + dcpen + housing)/1000
	
	foreach s in 1 2 4 5 6 8 9 28 29 {
		scatter dataformatch simformatch if spec==`s' 
		graph export "$output\test_`s'.png", replace
		graph export "$output\test_`s'.pdf", replace
		}
	
	/*..............................................................*/
	/* How does the model perform in terms of components of wealth? */
	/*..............................................................*/
	
		gen 	datacash = nettotnhw_bu_s_w1 + nethw_bu_s_w1 if spec==2 | spec==6 | spec==12 
		replace datacash = nettotnhw_bu_s_w1                 if spec==3 | spec==4 | spec==7 | spec==8 | spec==13 | spec==14
		
		gen 	datapen = penwealth_bu_w1   if spec==2 | spec==3 | spec==4  | spec==12 | spec==13 | spec==14
		replace datapen = priDCwealth_bu_w1 if spec==6 | spec==7 | spec==8 
			
		gen datahous = nethw_bu_s_w1 if spec==4 | spec==8  | spec==14

		gen 	datanonpen = datacash
		replace datanonpen = datanonpen + datahous if datahous~=.
		
		gen nonpen = cash + housing
		
		foreach var in datacash datapen datahous datanonpen cash dcpen housing nonpen {
			replace `var' = `var'/1000
			}

		** QE: FIGURE 3 **
			* Cash 
			twoway (scatter  datacash cash if spec==4 & datacash<400 & datacash>=0 & cash<400, mcolor(black%20))           				   ///
				   (lowess  datacash cash if spec==4 & datacash<400 & datacash>=0  & cash<400, lcolor(black) lwidth(medium))            ///
				   (function y=x, range(0 400) lcolor(black) lcolor(gs5) lpattern(dash) lwidth(medium)),                                      ///
				   scheme(s1color) aspect(1)                                                                                            	   ///
				   ylabel(0(100)400) yscale(range(0 400)) ytitle("Data: non-pension wealth (£000s)")                                      ///
				   xlabel(0(100)400, angle(0)) xscale(range(0 400)) xtitle("Model: non-pension wealth (£000s)")                            ///
				   /*legend(order(2 3) label(2 "Smoothed relationship") label(3 "45 degree line") region(lstyle(none)) cols(1) pos(12) ring(1))*/ ///
				   legend(off) 
			graph export "$output\comparison_cash_4_black.png", replace	   
			graph export "$output\comparison_cash_4_black.pdf", replace
			
			* Pension 
			twoway (scatter   datapen dcpen if spec==4 & datapen<400 & dcpen<400, mcolor(black%20))           				   ///
				   (lowess   datapen dcpen if spec==4 & datapen<400 & dcpen<400, lcolor(black) lwidth(medium))                   ///
				   (function y=x, range(0 400) lcolor(black) lcolor(gs5) lpattern(dash) lwidth(medium)),                                      ///
				   scheme(s1color) aspect(1)                                                                                            		   ///
				   ylabel(0(100)400) yscale(range(0 400)) ytitle("Data: pension wealth (£000s)")                                      ///
				   xlabel(0(100)400, angle(0)) xscale(range(0 400)) xtitle("Model: pension wealth (£000s)")                           ///
				   /*legend(order(2 3) label(2 "Smoothed relationship") label(3 "45 degree line") region(lstyle(none)) cols(1) pos(12) ring(1))*/  ///
				   legend(off) 
			graph export "$output\comparison_pension_4_black.png", replace
			graph export "$output\comparison_pension_4_black.pdf", replace
			
			* Non-pension 
			twoway (scatter  datanonpen nonpen if spec==4 & datanonpen<400 & datanonpen>=0 & nonpen<400, mcolor(black%20))           				   ///
				   (lowess  datanonpen nonpen if spec==4 & datanonpen<400 & datanonpen>=0  & nonpen<400, lcolor(black) lwidth(medium))            ///
				   (function y=x, range(0 400) lcolor(black) lcolor(gs5) lpattern(dash) lwidth(medium)),                                      ///
				   scheme(s1color) aspect(1)                                                                                            	   ///
				   ylabel(0(100)400) yscale(range(0 400)) ytitle("Data: non-pension wealth (£000s)")                                      ///
				   xlabel(0(100)400, angle(0)) xscale(range(0 400)) xtitle("Model: non-pension wealth (£000s)")                            ///
				   /*legend(order(2 3) label(2 "Smoothed relationship") label(3 "45 degree line") region(lstyle(none)) cols(1) pos(12) ring(1))*/ ///
				   legend(off) 
			graph export "$output\comparison_nonpen_4_black.png", replace	   
			graph export "$output\comparison_nonpen_4_black.pdf", replace	 
			
		
					
		*Portfolio shares
		gen datasharecash    = nettotnhw_bu_s_w1/(nettotnhw_bu_s_w1 + penwealth_bu_w1 + nethw_bu_s_w1) if spec==4 | spec==14 | spec==2 | spec==12
		gen datasharepen     = penwealth_bu_w1  /(nettotnhw_bu_s_w1 + penwealth_bu_w1 + nethw_bu_s_w1) if spec==4 | spec==14 | spec==2 | spec==12
		gen datasharenonpen  = (nettotnhw_bu_s_w1+nethw_bu_s_w1)  /(nettotnhw_bu_s_w1 + penwealth_bu_w1 + nethw_bu_s_w1) if spec==4 | spec==14 | spec==2 | spec==12
		gen modelsharecash   = cash             /(cash+dcpen+housing)								   if spec==4 | spec==14 | spec==2 | spec==12
		gen modelsharepen    = dcpen			/(cash+dcpen+housing)								   if spec==4 | spec==14 | spec==2 | spec==12
		gen modelsharenonpen = nonpen           /(cash+dcpen+housing)								   if spec==4 | spec==14 | spec==2 | spec==12
		
		replace datasharecash    = nettotnhw_bu_s_w1/(nettotnhw_bu_s_w1 + penwealth_bu_w1) if spec==3 | spec==13
		replace datasharepen     = penwealth_bu_w1  /(nettotnhw_bu_s_w1 + penwealth_bu_w1) if spec==3 | spec==13 
		replace datasharenonpen  = (nettotnhw_bu_s_w1)/(nettotnhw_bu_s_w1 + penwealth_bu_w1) if spec==3 | spec==13 
		replace modelsharecash   = cash             /(cash+dcpen)						   if spec==3 | spec==13
		replace modelsharepen    = dcpen			/(cash+dcpen)						   if spec==3 | spec==13
		replace modelsharenonpen = nonpen           /(cash+dcpen)						   if spec==3 | spec==13

		
		** QE: TABLE 5  **
		label define earndlab 1 "Lowest" 2 "2" 3 "3" 4 "4" 5 "5" 6 "6" 7 "7" 8 "8" 9 "9" 10 "Highest", replace
		label values LTearnd earndlab
		capture gen all= "All"
			tabout LTearnd if spec==4 using "$output\obsmodwealth_byLTearn1b.tex", sum c(mean dataformatch mean simformatch mean datapen mean dcpen mean datanonpen mean nonpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
			tabout all      if spec==4 using "$output\obsmodwealth_byLTearn2b.tex", sum c(mean dataformatch mean simformatch mean datapen mean dcpen mean datanonpen mean nonpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
			tabout LTearnd if spec==4 using "$output\obsmodwealth_byLTearn1bmed.tex", sum c(median dataformatch median simformatch median datapen median dcpen median datanonpen median nonpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
			tabout all      if spec==4 using "$output\obsmodwealth_byLTearn2bmed.tex", sum c(median dataformatch median simformatch median datapen median dcpen median datanonpen median nonpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)

		preserve
		keep idauniq spec all LTearnd datasharepen datasharenonpen modelsharepen modelsharenonpen
		reshape wide LTearnd datasharepen datasharenonpen modelsharepen modelsharenonpen, i(idauniq) j(spec)
		
		** QE: FIGURE 9 **
		tabout LTearnd4 using "$output\portshares_byLTearn_2v3v4.tex", sum c(median datasharepen2 median modelsharepen2 median modelsharepen4 median datasharepen3 median modelsharepen3 ) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(2)	
		tabout all using "$output\portshares_byLTearn_2v3v4b.tex", sum c(median datasharepen2 median modelsharepen2 median modelsharepen4 median datasharepen3 median modelsharepen3 ) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(2)	

		
		
	/*................................*/
	/* Distribution of discount rates */
	/*................................*/

	** QE TABLE 4**
	capture gen speclabelled = spec
	label define speclabel 1 "one asset" 2 "no housing treatment" 4 "modeled housing", modify
	label values speclabelled speclabel
	foreach spec in 1 2 4 {	
		tabout speclabelled if spec==`spec' using "$output\betadist_`spec'.tex", sum c(mean beta p10 beta p25 beta median beta p75 beta p90 beta) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)
		}
	
	** QE FIGURE 2 **
	graph twoway (kdensity beta if spec==1 & beta>=0.8 & beta<=1.2, lcol(black)) (kdensity beta if spec==2 & beta>=0.8 & beta<=1.2, lcol(black) lpattern(dash))(kdensity beta if spec==4 & beta>=0.8 & beta<=1.2, lcol(black) lpattern(longdash_dot)), ///
			xtitle("Estimated discount factor") legend(order(1 "one asset" 2 "simple two asset" 3 "modeled housing") col(1) ring(0) pos(2) region(lstyle(none))) scheme(s1color) 
	graph export "$output\betacomparison_baseruns2.png", replace	
	graph export "$output\betacomparison_baseruns2.pdf", replace
	
	graph twoway (kdensity beta if spec==1 & beta>=0.8 & beta<=1.2, lcol(red)) (kdensity beta if spec==2 & beta>=0.8 & beta<=1.2, lcol(dkgreen)) (kdensity beta if spec==3 & beta>=0.8 & beta<=1.2, lcol(black)) (kdensity beta if spec==4 & beta>=0.8 & beta<=1.2, lcol(blue)) ///
				 (kdensity beta if spec==5 & beta>=0.8 & beta<=1.2, lcol(red) lpattern(dash)) (kdensity beta if spec==6 & beta>=0.8 & beta<=1.2, lcol(dkgreen) lpattern(dash)) (kdensity beta if spec==7 & beta>=0.8 & beta<=1.2, lcol(black) lpattern(dash)) (kdensity beta if spec==8 & beta>=0.8 & beta<=1.2, lcol(blue) lpattern(dash)), ///
			xtitle("Estimated discount factor") legend(order(1 "one asset" 2 "no housing treatment" 3 "data excludes housing" 4 "modelled housing") col(1) ring(0) pos(2) region(lstyle(none))) scheme(s1color) 
		
	* Correlation in betas across models
	preserve
		keep idauniq beta spec
		reshape wide beta, i(idauniq) j(spec)
		pwcorr beta1 beta2 beta3 beta4 beta9 beta28 beta29
			foreach spec in 1 2 3 4 8 9 28 29 {
				corr beta4 beta`spec'
				gen corrbeta4beta`spec' = r(rho)
				}
			gen all = "All"
			tabout all using "$output\corrwithbeta4.tex", sum c(mean corrbeta4beta1 mean corrbeta4beta2 mean corrbeta4beta3 mean corrbeta4beta4 mean corrbeta4beta8 mean corrbeta4beta9 mean corrbeta4beta28 mean corrbeta4beta29) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(4)
	restore
	

	/*..............................*/
	/* Validation of discount rates */
	/*..............................*/

	** QE: FIGURE 4 **
		* Education (w1)
			replace schleave=. if schleave<0
		graph twoway (kdensity beta if schleave==0 & spec==4, lcol(black) range(0.8 1.2)) (kdensity beta if schleave==1 & spec==4, lcol(black) lpattern(dash) range(0.8 1.2)) (kdensity beta if schleave==2 & spec==4, lcol(black) lpattern(shortdash_dot) range(0.8 1.2) legend(order(1 "High school drop out" 2 "High school" 3 "Some college") cols(1) pos(1) ring(0) region(lstyle(none)))), scheme(s1color) xtitle("Estimated discount factor") ytitle("Density")
		graph export "$output\beta_educ_4.png", replace
		graph export "$output\beta_educ_4.pdf", replace
		tabstat beta if spec==4, by(schleave) stat(n mean median sd)

		* Self-reported planning horizon (w1)
			tabstat beta if spec==4, by(explan) stat(n mean p10 p25 p50 p75 p90)
			recode explan (7 9 1 = 1) (2 3 = 2) (4 = 3) (5 6 = 4) , gen(planhorizon2)
			replace planhorizon2 = . if explan<0 | explan>9
			label define horizon2 1 "Few weeks if at all" 2 "Up to yr" 3 "Nxt few yrs" 4 "Next 5yrs+"
			label values planhorizon2 horizon2
		graph twoway (kdensity beta if planhorizon2==1 & spec==4, lcol(red) range(0.8 1.2)) (kdensity beta if planhorizon2==2 & spec==4, lcol(dkgreen) range(0.8 1.2)) (kdensity beta if planhorizon2==3 & spec==4, lcol(blue) range(0.8 1.2)) (kdensity beta if planhorizon2==4 & spec==4, lcol(pink) range(0.8 1.2)), legend(order(1 "Few weeks if at all" 2 "Up to yr" 3 "Nxt few yrs" 4 "Next 5yrs+") cols(1) pos(2) ring(0)) scheme(s1color) 		
		graph export  "$output\beta_plan_4.png", replace
		graph export  "$output\beta_plan_4.pdf", replace
		
		graph twoway (kdensity beta if spec==4 & beta>=0.8 & beta<=1.2 & (planhorizon2==1|planhorizon2==2), lcolor(black) xscale(range(0.8 1.2))) (kdensity beta if spec==4 & beta>=0.8 & beta<=1.2 & (planhorizon2==3|planhorizon2==4), lcolor(black) lpattern(dash) xscale(range(0.8 1.2))), ///
			xtitle("Estimated discount factor") xlabel(0.8(0.1)1.2) ytitle("Density") legend(order(1 "Less than a year" 2 "Next few years or longer") col(1) ring(0) pos(2) region(lstyle(none))) scheme(s1color) 
		graph export  "$output\beta_plan2_4.png", replace
		graph export  "$output\beta_plan2_4.pdf", replace

	** QE: TABLE 6 **
		* Multivariate
			*-Education
			replace schleave=. if schleave<0
			*-Smoker
			recode smokerstat (0 = 0) (1 2 3 = 1) (4 = 2), gen(smoker2)
			replace smoker2 = . if smokerstat<0
			label define smoker2 0 "Never" 1 "Ex-smoker" 2 "Current"
			label values smoker2 smoker2
			*-Self-reported life expectancy
			gen 	exlogrp = 1 if exlo80>=0 & exlo80<50
			replace exlogrp = 2 if exlo80==50
			replace exlogrp = 3 if exlo80>50 & exlo80<=70
			replace exlogrp = 4 if exlo80>70 & exlo80<.
		
		forv spec = 1/4 {
			qreg beta i.planhorizon2 if spec==`spec'
			outreg2 using "$output\betaqregs_`spec'.tex", tex(frag) se bdec(3) replace
			qreg beta i.schleave if spec==`spec'
			outreg2 using "$output\betaqregs_`spec'.tex", tex(frag) se bdec(3) 
			qreg beta i.smoker2 if spec==`spec'
			outreg2 using "$output\betaqregs_`spec'.tex", tex(frag) se bdec(3) 
			qreg beta i.planhorizon2 i.schleave i.smoker2 if spec==`spec'
			outreg2 using "$output\betaqregs_`spec'.tex", tex(frag) se bdec(3) 
			qreg beta i.planhorizon2 i.schleave i.smoker2 i.exlogrp if exlo80>=0 & exlo80<. & spec==`spec'
			outreg2 using "$output\betaqregs_`spec'.tex", tex(frag) se bdec(3) 
			}
		}
		

	/*...........................*/
	/* Scholz et al. comparisons */
	/*...........................*/

	gen 	totalw     = (cash + dcpen + housing) 							           if spec==11 | spec==12 | spec==14 | spec==15 | spec==16 | spec==18
	replace totalw     = (cash + dcpen)           		                               if spec==13 | spec==17
	gen 	datatotalw = (nettotnhw_bu_s_w1 + penwealth_bu_w1   + nethw_bu_s_w1)/1000  if spec==11 | spec==12 | spec==14
	replace datatotalw = (nettotnhw_bu_s_w1 + penwealth_bu_w1                  )/1000  if spec==13
	replace datatotalw = (nettotnhw_bu_s_w1 + priDCwealth_bu_w1 + nethw_bu_s_w1)/1000  if spec==15 | spec==16 | spec==18
	replace datatotalw = (nettotnhw_bu_s_w1 + priDCwealth_bu_w1                )/1000  if spec==17
	gen oversaving = (datatotalw>totalw)*100 if totalw~=. & datatotalw~=.
	gen c_surplus    =  datatotalw - totalw              if oversaving==100 & (spec==11 | spec==12 | spec==13 | spec==14 | spec==15 | spec==16 | spec==17 | spec==18)
	gen c_surplus_p  = (datatotalw - totalw)/totalw*100  if oversaving==100 & (spec==11 | spec==12 | spec==13 | spec==14 | spec==15 | spec==16 | spec==17 | spec==18)
	gen c_deficit    =  totalw - datatotalw              if oversaving==0   & (spec==11 | spec==12 | spec==13 | spec==14 | spec==15 | spec==16 | spec==17 | spec==18)
	gen rsq_m=.
	
	** QE: FIGURE 5 (s=11, s=12, s=14) & QE FIGURE 10 (s=14, s=13) & QE: FIGURE 13 (s=15, s=16, s=18)**
	foreach s in 11 12 13 14 15 16 18 {
		* Total wealth scatters
		twoway (scatter datatotalw totalw if spec==`s' & totalw<800 & datatotalw<800 & datatotalw>0, mcolor(black%20))           				   ///
			   (lowess datatotalw totalw if spec==`s' & totalw<800 & datatotalw<800 & datatotalw>0, lcolor(black) lwidth(medium))            ///
			   (function y=x, range(0 800) lcolor(black) lcolor(gs5) lpattern(dash) lwidth(medium)),                                      ///
				scheme(s1color) aspect(1)                                                                                            	   ///
				ylabel(0(100)800) yscale(range(0 800)) ytitle("Data: total (£000s)")                                     ///
				xlabel(0(100)800, angle(0)) xscale(range(0 800)) xtitle("Model: total wealth (£000s)")                           ///
				/*legend(order(2 3) label(2 "Smoothed relationship") label(3 "45 degree line") region(lstyle(none)) cols(1) pos(12) ring(1))*/ ///
				legend(off) 
		graph export "$output\scholz_`s'_black.png", replace	   
		graph export "$output\scholz_`s'_black.pdf", replace	
		* R-squared
		reg totalw datatotalw if spec==`s' & totalw<1000 & datatotalw<1000
		scalar rsq_m`s' = e(r2)
		replace rsq_m =  rsq_m`s' if spec==`s'
		}

	capture gen speclabelled = spec
	label define speclabel 11 "One asset" 12 "Simple two asset" 13 "13" 14 "Modeled housing" 15 "One asset (exog DB)" 16 "Simple two asset (exog DB)" 17 "17" 18 "Modeled housing (exog DB)", modify
	label values speclabelled speclabel
	
	** QE: TABLE 8 **
	tabout speclabelled if spec==11 | spec==12 | spec==14 using "$output\Scholz_comparisons.tex", sum c(median datatotalw median totalw mean oversaving median c_surplus median c_deficit mean rsq_m) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1 1 1 1 1 3)
	
	** QE TABLE 11 **
	tabout speclabelled if spec==11 | spec==12 | spec==13 | spec==14 using "$output\Scholz_comparisons_all.tex", sum c(median datatotalw median totalw mean oversaving median c_surplus median c_deficit mean rsq_m) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1 1 1 1 1 3)

	
	/*...............................................*/
	/* Observed wealth - by lifetime earnings decile */
	/*...............................................*/

	** QE: TABLE 2 **
	label define earndlab 1 "Lowest" 2 "2" 3 "3" 4 "4" 5 "5" 6 "6" 7 "7" 8 "8" 9 "9" 10 "Highest", replace
	label values LTearnd earndlab
	capture gen all= "All"
	replace LTearn=LTearn/1000
	tabout LTearnd if spec==14 using "$output\obswealth_byLTearn_tab1.tex", sum c(median LTearn mean LTearn median dataformatch mean dataformatch median datapen mean datapen median datacash mean datacash median datahous mean datahous) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
	tabout all      if spec==14 using "$output\obswealth_byLTearn_tab2.tex", sum c(median LTearn mean LTearn median dataformatch mean dataformatch median datapen mean datapen median datacash mean datacash median datahous mean datahous) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
	tabout LTearnd if spec==14 using "$output\obswealth_byLTearn_tab1b.tex", sum c(median dataformatch mean dataformatch median datapen mean datapen median datacash mean datacash median datahous mean datahous) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
	tabout all      if spec==14 using "$output\obswealth_byLTearn_tab2b.tex", sum c(median dataformatch mean dataformatch median datapen mean datapen median datacash mean datacash median datahous mean datahous) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)

	
	/*...........................................................*/
	/* Correlation between estimated betas and replacement rates */
	/*...........................................................*/
	
		merge m:1 idauniq using "$data\RRdata.dta"
		assert _m~=1

		** QE: FIGURE 6 **
		replace retinc2_av  = retinc2_av*100
		replace retinc2_5yr = retinc2_5yr*100
		twoway (kdensity retinc2_av if retinc2_av>0 & retinc2_av<200, lcolor(black) lpattern(solid)) ///
			(kdensity retinc2_5yr if retinc2_5yr>0 & retinc2_5yr<200, lcolor(black) lpattern(dash)), ///
			xlabel(0(20)200, grid) xscale(range(0 200)) xtitle("Replacement rate %") ytitle("Density") ylabel(, angle(0)) ///
			legend(label(1 "of average lifetime earnings") label(2 "of average best 5yrs earnings") pos(1) col(1) ring(0) region(style(none))) ///
			scheme(s1color)
		graph export "$output\penRR_kdensity_av5yr.png", replace
		graph export "$output\penRR_kdensity_av5yr.pdf", replace

		keep if _m==3
		drop _m
	
	** QE: TABLE 9 **
	tabout all if spec==4 using "$output\RRbybetabins.tex", sum c(p50 retinc2_av p50 retinc3_av p50 retinc4_av p50 retinc2_5yr p50 retinc3_5yr p50 retinc4_5yr) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)	
	tabout beta_d if spec==4 using "$output\RRbybetabinsb.tex", sum c(p50 retinc2_av p50 retinc3_av p50 retinc4_av p50 retinc2_5yr p50 retinc3_5yr p50 retinc4_5yr) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)	
	
		
	/*...........................................*/
	/* Examining different ways of doing housing */
	/*...........................................*/
	
	* Beta summary table
	** QE: TABLE 10 **
	capture gen speclabelled = spec
	label define speclabel 2 "Simple two asset" 4 "Modeled housing" 3 "Housing excluded", modify
	label values speclabelled speclabel
	foreach spec in 2 3 4 {
		tabout speclabelled if spec==`spec' using "$output\betadist_`spec'h.tex", sum c(mean beta p10 beta p25 beta median beta p75 beta p90 beta) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)
		}

	* Beta scatter graph
	** QE: FIGURE 8 **
	preserve
	keep idauniq beta spec
	reshape wide beta, i(idauniq) j(spec)
		twoway (scatter beta3 beta4 if beta3>0.8 & beta3<1.1 & beta4>0.8 & beta4<1.1, mcolor(black%20)), ///
			xtitle("Est. discount factor - modeled housing") ytitle("Est. discount factor - housing excluded") ///
			xlabel(0.8(0.1)1.1) xscale(range(0.8 1.1)) ylabel(0.8(0.1)1.1, angle(0)) yscale(range(0.8 1.1)) ///
			 scheme(s1color) 
			graph export "$output\betascatter_3_4.png", replace	
			graph export "$output\betascatter_3_4.pdf", replace	
	restore


	/*.........................................*/
	/* Examining effct of having DB exogenous  */
	/*.........................................*/

	* Beta summary table
	** QE: TABLE 12 **
	capture gen speclabelled = spec
	label define speclabel 2 "Simple two asset" 6 "... with exogenous DB" 4 "Modeled housing" 8 "... with exogenous DB", modify
	label values speclabelled speclabel
	foreach spec in 2 6 4 8 {
		tabout speclabelled if spec==`spec' using "$output\betadist_`spec'exDB.tex", sum c(mean beta p10 beta p25 beta median beta p75 beta p90 beta) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(3)
		}

	* Beta scatter graph
	** QE: FIGURE 12 **
	preserve
	keep idauniq beta spec
	reshape wide beta, i(idauniq) j(spec)
	twoway (scatter beta8 beta4 if beta4>0.8 & beta4<1.1 & beta8>0.8 & beta8<1.1, mcolor(black%20)), ///
		xtitle("Est. discount factor - endogenous DB") ytitle("Est. discount factor - exogenous DB") ///
		xlabel(0.8(0.1)1.1) xscale(range(0.8 1.1)) ylabel(0.8(0.1)1.1, angle(0)) yscale(range(0.8 1.1)) ///
		 scheme(s1color) 
	graph export "$output\betascatter_8_4.png", replace	
	graph export "$output\betascatter_8_4.pdf", replace	
	restore
		
	** QE FIGURE 14 **
	twoway (lowess beta retinc2_av if spec==4 & retinc2_av<1.5 & beta>=0.85 & beta<=1.05 & retinc2_av>=0.25, yaxis(1) lcolor(black) lwidth(medium)) ///
		   (lowess beta retinc2_av if spec==8 & retinc2_av<1.5 & beta>=0.85 & beta<=1.05 & retinc2_av>=0.25, yaxis(1) lcolor(black) lpattern(dash) lwidth(medium)) ///
		   (kdensity retinc2_av if spec==4 & retinc2_av<1.5 & retinc2_av>=0.25, lcolor(black) lpattern(dot) yaxis(2)), ///
		    scheme(s1color) ylabel(0.85(0.05)1.05) yscale(range(0.85 1.05)) ytitle("Estimated discount factor", axis(1)) ytitle("Density", axis(2)) xscale(range(0.25 1.5)) xlabel(0.25(0.25)1.5) xtitle("Replacement rate") ///
			legend(order(1 2) label(1 "Baseline") label(2 "... with exogenous DB") region(lstyle(none)) cols(1) pos(4) ring(0))
	graph export "$output\RR2av_exDB.png", replace
	graph export "$output\RR2av_exDB.pdf", replace

	* Portfolio split - is done above for spec 8 
	
/*--------------------------------*/
/* Analysis that just uses age 64 */
/*--------------------------------*/

use "$data\data_for_analysis_all_elsa.dta", replace

* Drop those with truncated discount rates
drop if (beta<=0.49 | beta==1.5) 

keep if age_m == 64

save "$data\data_for_analysis_all_elsa_64.dta", replace
use "$data\data_for_analysis_all_elsa_64.dta", replace

	gen simformatch = (cash + dcpen + housing)/1000
	
		gen nonpen = cash + housing
		
		foreach var in cash dcpen housing nonpen {
			replace `var' = `var'/1000
		}

	
	/*.........................*/
	/* Counterfactual analysis */
	/*.........................*/
	
		* Tables - moments of distributions of wealth
		capture gen speclabelled = spec
		label define speclabel 4 "Baseline" 23 "No pension tax adv"  26 "No pension asset" 40 "No pension (no housing), modify
		label values speclabelled speclabel
		capture gen data= "Data"

			* Pension tax advantgae and DC returns
			** QE: TABLE 7 **
			tabout speclabelled if (spec==4 | spec==23 | spec==26 | spec==40) using "$output\counterfactual64_total_taxadv.tex", sum c(p10 simformatch p25 simformatch p50 simformatch p75 simformatch p90 simformatch mean simformatch) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
			tabout speclabelled if (spec==4 | spec==23 | spec==26 | spec==40) using "$output\counterfactual64_pension_taxadv.tex", sum c(p10 dcpen p25 dcpen p50 dcpen p75 dcpen p90 dcpen mean dcpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)
			tabout speclabelled if (spec==4 | spec==23 | spec==26 | spec==40) using "$output\counterfactual64_nonpen_taxadv.tex", sum c(p10 nonpen p25 nonpen p50 nonpen p75 nonpen p90 nonpen mean nonpen) style(tex) replace lines(none) h1(nil) h2(nil) h3(nil) ptotal(none) f(1)

			
